
from epyk.core.html import tables as html_tables


class Plotly(object):
  def __init__(self, context):
    self.parent = context

  def table(self, records, cols=None, rows=None, header=None, width=(100, '%'), height=(None, 'px'), htmlCode=None, options=None, profile=None):
    """
    Description:
    ------------

    Usage::

      Related Pages:

			https://plot.ly/javascript/table-subplots/

    Attributes:
    ----------
    :param records:
    :param cols:
    :param rows:
    :param header:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile: 
    """
    data_rows, _header = [], []
    cols = cols or []
    rows = rows or []
    if len(records) > 0 and not cols and not rows:
      cols = list(records[0].keys())
    for r in rows:
      data_rows.append([])
      _header.append(r)
    data_cols = []
    for r in cols:
      data_cols.append([])
      _header.append(r)
    for rec in records:
      for i, r in enumerate(rows):
        data_rows[i].append(rec.get(r, ''))
      for i, r in enumerate(cols):
        data_cols[i].append(rec.get(r, ''))

    header = header or _header
    options = options or {}
    options.update({'type': 'table', 'mode': ''})
    h_table = html_tables.HtmlTablePlotly.Table(self.parent.context.rptObj, width, height, options, htmlCode, profile)
    h_table.add_trace(data_rows + data_cols)
    h_table.options.responsive = True
    h_table.data.header.values = [[h] for h in header]
    h_table.layout.no_background()
    return h_table



"""
packaging module to install: python.exe -m pip install --upgrade setuptools wheel
to create the tar.gz: python.exe setup.py sdist
to create the weels: python.exe setup.py sdist bdist_wheel --universal
"""

import setuptools
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

def install_required():
  return [line for line in open('requirements.txt')]

setuptools.setup(
    name="epyk",
    author="epykure",
    version="1.0.13",
    author_email="smith.pyotr@gmail.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/epykure/epyk-ui",
    project_urls={
        "Documentation": "http://www.epyk.io",
        "Code": "https://github.com/epykure/epyk-ui",
        "Issue tracker": "https://github.com/epykure/epyk-ui/issues"
    },
    packages=setuptools.find_packages(),
    install_requires=install_required(),
    package_data={'epyk': [os.path.join('static', 'images', '*'), os.path.join('static', 'images', 'logo', '*')]},
    entry_points={"console_scripts": ["epyk = epyk.core.cli.command_line_fncs:main"]},
    python_requires=">=2.7",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
)



from epyk.core.data.DataClass import DataClass
from epyk.core.js.packages import packageImport


class OptionAxesTicks(DataClass):

  @property
  def fontColor(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["fontColor"]

  @fontColor.setter
  def fontColor(self, val):
    self._attrs["fontColor"] = val

  @property
  def fontSize(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["fontSize"]

  @fontSize.setter
  def fontSize(self, val):
    self._attrs["fontSize"] = val

  @property
  def beginAtZero(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["beginAtZero"]

  @beginAtZero.setter
  def beginAtZero(self, val):
    self._attrs["beginAtZero"] = val

  @property
  def max(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["max"]

  @max.setter
  def max(self, val):
    self._attrs["max"] = val

  @property
  def min(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["min"]

  @min.setter
  def min(self, val):
    self._attrs["min"] = val

  @property
  def suggestedMin(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["suggestedMin"]

  @suggestedMin.setter
  def suggestedMin(self, val):
    self._attrs["suggestedMin"] = val

  @property
  def suggestedMax(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["suggestedMax"]

  @suggestedMax.setter
  def suggestedMax(self, val):
    self._attrs["suggestedMax"] = val

  @property
  def stepSize(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["stepSize"]

  @stepSize.setter
  def stepSize(self, val):
    self._attrs["stepSize"] = val


class OptionLabels(DataClass):

  @property
  def fontColor(self):
    """
    """
    return self._attrs["fontColor"]

  @fontColor.setter
  def fontColor(self, val):
    self._attrs["fontColor"] = val


class OptionAxesGridLine(DataClass):

  @property
  def display(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["display"]

  @display.setter
  def display(self, val):
    self._attrs["display"] = val

  @property
  def circular(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["circular"]

  @circular.setter
  def circular(self, val):
    self._attrs["circular"] = val

  @property
  def color(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["color"]

  @color.setter
  def color(self, val):
    self._attrs["color"] = val

  @property
  def borderDash(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["borderDash"]

  @borderDash.setter
  def borderDash(self, val):
    self._attrs["borderDash"] = val

  @property
  def borderDashOffset(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["borderDashOffset"]

  @borderDashOffset.setter
  def borderDashOffset(self, val):
    self._attrs["borderDashOffset"] = val

  @property
  def lineWidth(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["lineWidth"]

  @lineWidth.setter
  def lineWidth(self, val):
    self._attrs["lineWidth"] = val

  @property
  def drawBorder(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["drawBorder"]

  @drawBorder.setter
  def drawBorder(self, val):
    self._attrs["drawBorder"] = val

  @property
  def drawOnChartArea(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["drawOnChartArea"]

  @drawOnChartArea.setter
  def drawOnChartArea(self, val):
    self._attrs["drawOnChartArea"] = val

  @property
  def drawTicks(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["drawTicks"]

  @drawTicks.setter
  def drawTicks(self, val):
    self._attrs["drawTicks"] = val

  @property
  def tickMarkLength(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["tickMarkLength"]

  @tickMarkLength.setter
  def tickMarkLength(self, val):
    self._attrs["tickMarkLength"] = val

  @property
  def zeroLineWidth(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["zeroLineWidth"]

  @zeroLineWidth.setter
  def zeroLineWidth(self, val):
    self._attrs["zeroLineWidth"] = val

  @property
  def zeroLineColor(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["zeroLineColor"]

  @zeroLineColor.setter
  def zeroLineColor(self, val):
    self._attrs["zeroLineColor"] = val

  @property
  def zeroLineBorderDash(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["zeroLineBorderDash"]

  @zeroLineBorderDash.setter
  def zeroLineBorderDash(self, val):
    self._attrs["zeroLineBorderDash"] = val

  @property
  def zeroLineBorderDashOffset(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["zeroLineBorderDashOffset"]

  @zeroLineBorderDashOffset.setter
  def zeroLineBorderDashOffset(self, val):
    self._attrs["zeroLineBorderDashOffset"] = val

  @property
  def offsetGridLines(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["offsetGridLines"]

  @offsetGridLines.setter
  def offsetGridLines(self, val):
    self._attrs["offsetGridLines"] = val

  @property
  def z(self):
    """
    https://www.chartjs.org/docs/latest/axes/styling.html#grid-line-configuration
    """
    return self._attrs["z"]

  @z.setter
  def z(self, val):
    self._attrs["z"] = val


class OptionAxesScaleLabel(DataClass):

  @property
  def display(self):
    return self._attrs["display"]

  @display.setter
  def display(self, val):
    self._attrs["display"] = val

  @property
  def fontColor(self):
    return self._attrs["fontColor"]

  @fontColor.setter
  def fontColor(self, val):
    self._attrs["fontColor"] = val

  @property
  def labelString(self):
    return self._attrs["labelString"]

  @labelString.setter
  def labelString(self, val):
    self._attrs["labelString"] = val


class OptionAxes(DataClass):

  @property
  def display(self):
    return self._attrs["display"]

  @display.setter
  def display(self, val):
    self._attrs["display"] = val

  @property
  def type(self):
    return self._attrs["type"]

  @type.setter
  def type(self, val):
    self._attrs["type"] = val

  @property
  def stacked(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["stacked"]

  @stacked.setter
  def stacked(self, val):
    self._attrs["stacked"] = val

  @property
  def id(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["id"]

  @id.setter
  def id(self, val):
    self._attrs["id"] = val

  @property
  def offset(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["offset"]

  @offset.setter
  def offset(self, val):
    self._attrs["offset"] = val

  @property
  def position(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["position"]

  @position.setter
  def position(self, val):
    self._attrs["position"] = val

  @property
  def ticks(self):
    """

    :return:
    """
    return self.sub_data("ticks", OptionAxesTicks)

  @property
  def gridLines(self):
    return self.sub_data("gridLines", OptionAxesGridLine)

  @property
  def scaleLabel(self):
    return self.sub_data("scaleLabel", OptionAxesScaleLabel)

  def add_label(self, text, color=None):
    self.scaleLabel.display = True
    self.scaleLabel.labelString = text
    if color is not None:
      self.scaleLabel.fontColor = color
    return self

  def category(self, vals):
    """

    :param vals:
    :return:
    """
    self._attrs["type"] = "category"
    self._attrs["labels"] = vals


class OptionScales(DataClass):

  def add_y_axis(self):
    return self.sub_data_enum("yAxes", OptionAxes)

  def y_axis(self, i=None):
    """

    :param i:
    :rtype: OptionAxes
    """
    if "yAxes" not in self._attrs:
      self.add_y_axis()

    if i is None:
      return self._attrs["yAxes"][-1]

    return self._attrs["yAxes"][i]

  def add_x_axis(self):
    return self.sub_data_enum("xAxes", OptionAxes)

  def x_axes(self, i=None):
    if "xAxes" not in self._attrs:
      self.add_x_axis()

    if i is None:
      return self._attrs["xAxes"][-1]

    return self._attrs["xAxes"][i]


class OptionPadding(DataClass):

  @property
  def left(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["left"]

  @left.setter
  def left(self, val):
    self._attrs["left"] = val

  @property
  def right(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["right"]

  @right.setter
  def right(self, val):
    self._attrs["right"] = val

  @property
  def top(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["top"]

  @top.setter
  def top(self, val):
    self._attrs["top"] = val

  @property
  def bottom(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["bottom"]

  @bottom.setter
  def bottom(self, val):
    self._attrs["bottom"] = val


class OptionLayout(DataClass):

  @property
  def padding(self):
    return self.sub_data("padding", OptionPadding)


class OptionLegend(DataClass):

  @property
  def labels(self):
    return self.sub_data("labels", OptionLabels)


class OptionTitle(DataClass):
  @property
  def display(self):
    """
    https://www.chartjs.org/docs/latest/configuration/title.html

    """
    return self._attrs["display"]

  @display.setter
  def display(self, val):
    self._attrs["display"] = val

  @property
  def text(self):
    return self._attrs["text"]

  @text.setter
  def text(self, val):
    self._attrs["text"] = val

  @property
  def position(self):
    return self._attrs["position"]

  @position.setter
  def position(self, val):
    self._attrs["position"] = val

  @property
  def fontSize(self):
    return self._attrs["fontSize"]

  @fontSize.setter
  def fontSize(self, val):
    self._attrs["fontSize"] = val

  @property
  def fontFamily(self):
    return self._attrs["fontFamily"]

  @fontFamily.setter
  def fontFamily(self, val):
    self._attrs["fontFamily"] = val

  @property
  def fontColor(self):
    return self._attrs["fontColor"]

  @fontColor.setter
  def fontColor(self, val):
    self._attrs["fontColor"] = val

  @property
  def fontStyle(self):
    return self._attrs["fontStyle"]

  @fontStyle.setter
  def fontStyle(self, val):
    self._attrs["fontStyle"] = val

  @property
  def padding(self):
    return self._attrs["padding"]

  @padding.setter
  def padding(self, val):
    self._attrs["padding"] = val

  @property
  def lineHeight(self):
    return self._attrs["lineHeight"]

  @lineHeight.setter
  def lineHeight(self, val):
    self._attrs["lineHeight"] = val


class Options(DataClass):

  @property
  def scales(self):
    return self.sub_data("scales", OptionScales)

  @property
  def layout(self):
    return self.sub_data("layout", OptionLayout)

  @property
  def title(self):
    return self.sub_data("title", OptionTitle)

  @property
  def legend(self):
    return self.sub_data("legend", OptionLegend)

  @property
  def plugins(self):
    return self.sub_data("plugins", OptionChartJsPlugins)

  def add_title(self, text, color=None):
    """

    :param text:
    :param color:
    """
    self.title.display = True
    self.title.text = text
    if color is not None:
      self.title.fontColor = color
    return self

  @property
  def managed(self):
    """
    Description:
    ------------
    """
    return self.get(True)

  @managed.setter
  def managed(self, bool):
    self.set(bool)


class OptionPieAnimation(DataClass):
  @property
  def animateRotate(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["animateRotate"]

  @animateRotate.setter
  def animateRotate(self, val):
    self.attr("animateRotate", val)

  @property
  def animateScale(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["animateScale"]

  @animateScale.setter
  def animateScale(self, val):
    self.attr("animateScale", val)


class OptionsBar(Options):
  pass


class OptionsPie(Options):

  @property
  def cutoutPercentage(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["cutoutPercentage"]

  @cutoutPercentage.setter
  def cutoutPercentage(self, val):
    self._attrs["cutoutPercentage"] = val

  @property
  def rotation(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["rotation"]

  @rotation.setter
  def rotation(self, val):
    self._attrs["rotation"] = val

  @property
  def circumference(self):
    """
    https://www.chartjs.org/docs/latest/charts/line.html
    """
    return self._attrs["circumference"]

  @circumference.setter
  def circumference(self, val):
    self._attrs["circumference"] = val

  @property
  def animation(self):
    return self.sub_data("animation", OptionPieAnimation)


class OptionsLine(Options):
  @property
  def showLines(self):
    """
    https://www.chartjs.org/docs/latest/charts/polar.html
    """
    return self._attrs["showLines"]

  @showLines.setter
  def showLines(self, val):
    self._attrs["showLines"] = val

  @property
  def spanGaps(self):
    """
    https://www.chartjs.org/docs/latest/charts/polar.html
    """
    return self._attrs["spanGaps"]

  @spanGaps.setter
  def spanGaps(self, val):
    self._attrs["spanGaps"] = val


class OptionsPolar(Options):

  @property
  def startAngle(self):
    """
    https://www.chartjs.org/docs/latest/charts/polar.html
    """
    return self._attrs["startAngle"]

  @startAngle.setter
  def startAngle(self, val):
    self._attrs["startAngle"] = val

  @property
  def animation(self):
    return self.sub_data("animation", OptionPieAnimation)


class OptionChartJsPlugins(DataClass):

  @property
  @packageImport('chartjs-plugin-zoom')
  def zoom(self):
    """
    Description:
    -----------
    A zoom and pan plugin for Chart.js. Currently requires Chart.js >= 2.6.0

    Related Pages:

      https://github.com/chartjs/chartjs-plugin-zoom
    """
    from epyk.core.html.graph.exts import ChartJsZoom
    return self.sub_data("zoom", ChartJsZoom.Zoom)

  @property
  @packageImport('chartjs-plugin-crosshair')
  def crosshair(self):
    """
    Description:
    -----------

    Related Pages:

      https://github.com/chartjs/chartjs-plugin-zoom
    """
    from epyk.core.html.graph.exts import ChartJsCrosshair
    return self.sub_data("crosshair", ChartJsCrosshair.Crosshair)

  @property
  @packageImport('chartjs-plugin-annotation')
  def annotation(self):
    """
    Description:
    -----------
    An annotation plugin for Chart.js >= 2.4.0

    This plugin draws lines and boxes on the chart area.

    Annotations work with line, bar, scatter and bubble charts that use linear, logarithmic, time, or category scales.
    Annotations will not work on any chart that does not have exactly two axes, including pie, radar, and polar area charts.

    Related Pages:

      https://github.com/chartjs/chartjs-plugin-zoom
    """
    from epyk.core.html.graph.exts import ChartJsAnnotation
    return self.sub_data("annotation", ChartJsAnnotation.Annotation)






from epyk.core.data.DataClass import DataClass


class DataBreakPoints(DataClass):

  @property
  def name(self):
    """
    Description:
    -----------

    Related Pages:

			https://datatables.net/reference/option/responsive.breakpoints
    """
    return self._attrs["name"]

  @name.setter
  def name(self, val):
    self._attrs["name"] = val

  @property
  def width(self):
    """
    Description:
    -----------

    Related Pages:

			https://datatables.net/reference/option/responsive.breakpoints
    """
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val


class DataDetails(DataClass):

  @property
  def type(self):
    """
    Description:
    -----------

    Related Pages:

			https://datatables.net/reference/option/responsive.details
    """
    return self._attrs["type"]

  @type.setter
  def type(self, val):
    self._attrs["type"] = val

  @property
  def display(self):
    """
    Description:
    -----------
    Responsive provides the ability to show information about the columns it has hidden using DataTables child rows feature (row().child()), but you may wish to display the data in a different manner (potentially so you can use the child rows for other actions such as editing) - this parameter provides that ability.

    Related Pages:

			https://datatables.net/reference/option/responsive.details.display
    """
    return self._attrs["display"]

  @display.setter
  def display(self, val):
    self._attrs["display"] = val

  @property
  def renderer(self):
    """
    Description:
    -----------


    Related Pages:

			https://datatables.net/reference/option/responsive.details.renderer
    """
    return self._attrs["renderer"]

  @renderer.setter
  def renderer(self, val):
    self._attrs["renderer"] = val

  @property
  def target(self):
    """
    Description:
    -----------

    Related Pages:

			https://datatables.net/reference/option/responsive.details.target
    """
    return self._attrs["target"]

  @target.setter
  def target(self, val):
    self._attrs["target"] = val


class Responsive(DataClass):

  def activate(self):
    self.details.type = "column"
    return self

  @property
  def details(self):
    """
    Description:
    -----------
    Responsive has the ability to use DataTables' child rows feature to show information about any columns which have been removed from the display as a child row, which can be particularly useful for displaying complex information on small screen devices. Please see the Responsive manual on the details rows for further information.

    Related Pages:

			https://datatables.net/reference/option/responsive.details

    :rtype: DataDetails
    """
    return self.sub_data("details", DataDetails)

  @property
  def breakpoints(self):
    """
    Description:
    -----------
    The visibility of columns in a DataTable with Responsive enabled can be controlled by breakpoints and class names matching those breakpoints (and other logical operations) .
    This provides the ability to exactly control which columns in a table will be visible for each device type. See the Responsive manual for more information.

    Related Pages:

			https://datatables.net/reference/option/responsive.breakpoints

    :rtype: DataBreakPoints
    """
    return self.sub_data("breakpoints", DataBreakPoints)

  @property
  def orthogonal(self):
    """
    Description:
    -----------
    When Responsive is asked to show the hidden information about a DataTable row,
    it uses the cell().render() method to access this data for each cell in the row.
    This method provides DataTables ability to make use of orthogonal data - that is,
    the same data but formatted in a different way for different uses.

    Related Pages:

			https://datatables.net/reference/option/responsive.details.target
    """
    return self._attrs["orthogonal"]

  @orthogonal.setter
  def orthogonal(self, val):
    self._attrs["orthogonal"] = val




from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsObjects

# All the predefined Javascript Statements
from epyk.core.js.statements import JsIf
from epyk.core.js.statements import JsErrors
from epyk.core.js.statements import JsFor
from epyk.core.js.statements import JsSwitch
from epyk.core.js.statements import JsWhile


def if_(jsCond, jsFnc):
  """
  Description:
  ------------
  Conditional statements are used to perform different actions based on different conditions.

  Related Pages:

    https://www.w3schools.com/js/js_if_else.asp

  Attributes:
  ----------
  :param jsCond:
  :param jsFnc:
  """
  if isinstance(jsCond, list):
    jsCond = "(%s)" % ")||(".join(JsUtils.jsConvertFncs(jsCond))
  return JsIf.JsIf(jsCond, jsFnc)


def switch(variable):
  """
  Description:
  ------------
  switch statement is used to perform different actions based on different conditions.

  Related Pages:

    https://www.w3schools.com/js/js_switch.asp

  Attributes:
  ----------
  :param variable:
  """
  if hasattr(variable, 'dom'):
    variable = variable.dom.content
  variable = JsUtils.jsConvertData(variable, None)
  return JsSwitch.JsSwitch(variable)


def while_(pivot, jsFnc=None, options=None):
  """
  Description:
  ------------

  Related Pages:

    https://www.w3schools.com/js/js_loop_while.asp

  Attributes:
  ----------
  :param pivot:
  :param jsFnc:
  :param options:
  """
  js_while = JsWhile.JsWhile(pivot, options=options)
  if jsFnc is not None:
    js_while.fncs(jsFnc)
  return js_while


def whileOf(jsIterable, jsFnc=None, options=None):
  """
  Description:
  ------------

  Related Pages:

    https://www.w3schools.com/js/js_loop_while.asp

  Attributes:
  ----------
  :param jsIterable:
  :param jsFnc:
  """
  if hasattr(jsIterable, 'dom'):
    jsIterable = jsIterable.dom.content
  js_for = JsWhile.JsWhileIterable(jsIterable, options=options)
  if jsFnc is not None:
    js_for.fncs(jsFnc)
  return js_for


def for_(end, jsFnc=None, options=None):
  """
  Description:
  ------------
  Loops can execute a block of code a number of times.

  Related Pages:

    https://www.w3schools.com/js/js_loop_for.asp

  Attributes:
  ----------
  :param end:
  :param jsFnc:
  """
  if hasattr(end, 'dom'):
    end = end.dom.content.number
  js_for = JsFor.JsFor(end, options=options)
  if jsFnc is not None:
    js_for.fncs(jsFnc)
  return js_for


def forIn(jsObj, jsFnc=None, options=None):
  """
  The JavaScript for/in statement loops through the properties of an object

  :param jsObj:
  :param jsFnc:
  """
  if hasattr(jsObj, 'dom'):
    jsObj = jsObj.dom.content
  js_for = JsFor.JsIterable(jsObj, options=options)
  if jsFnc is not None:
    js_for.fncs(jsFnc)
  return js_for


def forOf(jsIterable, jsFnc=None, options=None):
  """
  The JavaScript for/of statement loops through the values of an iterable objects

  :param jsIterable:
  :param jsFnc:
  """
  if hasattr(jsIterable, 'dom'):
    jsIterable = jsIterable.dom.content
  dflt_options = {"type": 'of'}
  if options is not None:
    dflt_options.update(options)
  js_for = JsFor.JsIterable(jsIterable, options=dflt_options)
  if jsFnc is not None:
    js_for.fncs(jsFnc)
  return js_for


def typeof(jsData, type=None):
  """
  Description:
  ------------
  The typeof function

  Related Pages:

    https://www.w3schools.com/js/js_datatypes.asp

  Attributes:
  ----------
  :param jsData: String. A String corresponding to a JavaScript object
  :param type: String. The type of object
  """
  if type is None:
    return JsObjects.JsBoolean.JsBoolean("typeof %s" % jsData)

  return JsObjects.JsVoid("typeof %s === '%s'" % (jsData, type))


def not_(jsCond):
  """
  Description:
  ------------

  Attributes:
  ----------
  :param jsCond:
  """
  return "!(%s)" % JsUtils.jsConvertData(jsCond, None)


def try_(jsFncs):
  """
  Description:
  ------------
  Block of code to try

  Related Pages:

    https://www.w3schools.com/js/js_errors.asp

  Attributes:
  ----------
  :param jsFncs: List.
  """
  if not isinstance(jsFncs, list):
    jsFncs = [jsFncs]
  return JsErrors.JsTry(jsFncs)


def and_(*args):
  """
  Description:
  ------------

  Attributes:
  ----------
  :param args:
  """
  return "(%s)" % ") && (".join([JsUtils.jsConvertData(x, None) for x in args])


def throw(value):
  """
  Description:
  ------------
  The throw statement allows you to create a custom error.

  Technically you can throw an exception (throw an error).

  Related Pages:

    https://www.w3schools.com/js/js_errors.asp

  Attributes:
  ----------
  :param value: String. The message displayed with the exception
  """
  return JsObjects.JsObject.JsObject.get("throw %s" % JsUtils.jsConvertData(value, None))


def or_(*args):
  """
  Description:
  ------------

  Attributes:
  ----------
  :param args:
  """
  return "(%s)" % ") || (".join([JsUtils.jsConvertData(x, None) for x in args])


break_ = JsObjects.JsObject.JsObject.get("break")


continue_ = JsObjects.JsObject.JsObject.get("continue")




# https://bl.ocks.org/ctufts/f38ef0187f98c537d791d24fda4a6ef9

from epyk.core.html import Html
from epyk.core.js import JsUtils


class Script(Html.Html):
  name = 'D3 Script'

  def __init__(self, report, data, width, height, htmlCode, options, profile):
    super(Script, self).__init__(report, data, htmlCode=htmlCode, css_attrs={"width": width, "height": height}, profile=profile)
    self.__loader = ""
    self._jsStyles.update(options)
    self._jsStyles.update({"wdith": width[0], "height": height[0]})

  def data(self, data):
    if not isinstance(data, list):
      data = data.split(" ")
    self._vals = data

  def loader(self, str_frg, pretty_format=False):
    #fgs = [l.lstrip() for l in str_frg.split()]
    self.__loader = str_frg

  def build(self, data=None, options=None, profile=False):
    if not self.builder_name:
      raise Exception("No builder defined for this HTML component %s" % self.__class__.__name__)

    constructors = self._report._props.setdefault("js", {}).setdefault("constructors", {})
    constructors[self.builder_name] = "function %s(htmlObj, data, options){%s}" % (
    self.builder_name, self.__loader)
    if isinstance(data, dict):
      # check if there is no nested HTML components in the data
      tmp_data = ["%s: %s" % (JsUtils.jsConvertData(k, None), JsUtils.jsConvertData(v, None)) for k, v in data.items()]
      js_data = "{%s}" % ",".join(tmp_data)
    else:
      js_data = JsUtils.jsConvertData(data, None)
    options, js_options = options or self._jsStyles, []
    for k, v in options.items():
      if isinstance(v, dict):
        row = ["'%s': %s" % (s_k, JsUtils.jsConvertData(s_v, None)) for s_k, s_v in v.items()]
        js_options.append("'%s': {%s}" % (k, ", ".join(row)))
      else:
        if self.options.isJsContent(k) or str(v).strip().startswith("function"):
          js_options.append("%s: %s" % (k, v))
        else:
          js_options.append("%s: %s" % (k, JsUtils.jsConvertData(v, None)))
    return "%s(%s, %s, %s)" % (self.builder_name, self.dom.d3.varId, js_data, "{%s}" % ",".join(js_options))

  def __str__(self):
    self._report._props.setdefault('js', {}).setdefault("builders", []).append(self.refresh())
    return '<div %s></div>' % self.get_attrs(pyClassNames=self.style.get_classes())




import os

from epyk.core.html.graph import GraphD3


class D3(object):

  def __init__(self, context):
    self.parent = context
    self.chartFamily = "D3"

  def script(self, name, scripts=None, data=None, d3_version=None, dependencies=None, profile=None, options=None, width=(400, "px"), height=(330, "px"), htmlCode=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param name:
    :param scripts:
    :param d3_version:
    :param dependencies:
    """
    if scripts is not None:
      scripts = [os.path.split(script) for script in scripts]

      dependencies = dependencies or []
      self.parent.context.rptObj.ext_packages = self.parent.context.rptObj.ext_packages or {}
      dependencies.append({'alias': 'd3', 'version': d3_version} if d3_version is not None else "d3")

      self.parent.context.rptObj.ext_packages[name] = {
          'version': '', 'req': [{'alias': d} if not isinstance(d, dict) else d for d in dependencies],
          'register': {'alias': name, 'module': scripts[0][1][:-3]},
          'modules': [{'script': split_url[1], 'path': '', 'cdnjs': split_url[0]} for split_url in scripts]}
      self.parent.context.rptObj.jsImports.add(name)
    else:
      self.parent.context.rptObj.jsImports.add("d3")
    d3_chart = GraphD3.Script(self.parent.context.rptObj, data or [], width, height, htmlCode, options or {}, profile)
    d3_chart.builder_name = "%s%s" % (d3_chart.builder_name, name)
    return d3_chart

  def cloud(self, data, width=(300, "px"), height=(330, "px"), htmlCode=None, options=None, profile=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param data:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    scripts = ["https://cdnjs.cloudflare.com/ajax/libs/d3-cloud/1.2.5/d3.layout.cloud.min.js"]
    chart = self.script('cloud', scripts, None, profile=profile, options=options, width=width, height=height, htmlCode=htmlCode)
    chart.loader('''
    var layout = d3.layout.cloud().size([options.wdith, options.height])
        .words(data.map(function(d) { return {text: d, size: 10 + Math.random() * 90}; }))
        .padding(5).rotate(function() { return ~~(Math.random() * 2) * 90; })
        .font("Impact").fontSize(function(d) { return d.size; }).on("end", draw);
    layout.start();

    function draw(words) {
      htmlObj.append("svg").attr("width", layout.size()[0]).attr("height", layout.size()[1])
        .append("g").attr("transform", "translate(" + layout.size()[0] / 2 + "," + layout.size()[1] / 2 + ")")
        .selectAll("text").data(words)
        .enter().append("text")
          .style("font-size", function(d) { return d.size + "px"; })
          .style("font-family", "Impact")
          .attr("text-anchor", "middle")
          .attr("transform", function(d) {
            return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")"})
          .text(function(d) { return d.text; });
    }''')
    chart.data(data)
    return chart




from epyk.core import html
from epyk.core.css import Defaults_css


class Titles(object):

  def __init__(self, context):
    self.context = context

  def head(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip, dflt_options, profile)
    html_title.style.css.color = self.context.rptObj.theme.colors[-1]
    html_title.style.css.border_left = '5px solid %s' % self.context.rptObj.theme.colors[-1]
    html_title.style.css.font_size = Defaults_css.font(15)
    html_title.style.css.text_transform = 'uppercase'
    html_title.style.css.margin_left = 2
    html_title.style.css.padding_left = 2
    html_title.style.css.bold()
    return html_title

  def headline(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip, dflt_options, profile)
    html_title.style.css.font_size = Defaults_css.font(4)
    html_title.style.css.color = self.context.rptObj.theme.colors[-1]
    html_title.style.css.font_style = 'italic'
    return html_title

  def title(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip, dflt_options, profile)
    html_title.style.css.font_size = Defaults_css.font(6)
    html_title.style.css.color = self.context.rptObj.theme.colors[-1]
    return html_title

  def rubric(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip, dflt_options, profile)
    html_title.style.css.border_left = '3px solid %s' % self.context.rptObj.theme.colors[-1]
    html_title.style.css.font_size = Defaults_css.font(6)
    return html_title

  def caption(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip, dflt_options, profile)
    html_title.style.css.font_size = Defaults_css.font(4)
    html_title.style.css.color = self.context.rptObj.theme.colors[-1]
    return html_title

  def underline(self, text=None, options=None, tooltip="", width=(None, "px"), height=('auto', ""), htmlCode=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param text:
    :param options:
    :param tooltip:
    :param width:
    :param height:
    :param htmlCode:
    :param profile:
    """
    dflt_options = {'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlTags.HtmlGeneric(self.context.rptObj, "div", text, width, height, htmlCode, tooltip,
                                           dflt_options, profile)
    html_title.style.css.font_size = Defaults_css.font(6)
    html_title.style.css.border_bottom = '2px solid %s' % self.context.rptObj.theme.colors[-1]
    return html_title



"""

"""

from epyk.core.css.catalogs import Catalog

from epyk.core.css.styles.classes import CssStylesIcon


class CatalogIcon(Catalog.CatalogGroup):
  def basic(self):
    """  """
    return self._set_class(CssStylesIcon.CssIcon)

  def standard(self):
    """  """
    return self._set_class(CssStylesIcon.CssStdIcon)

  def small(self):
    """  """
    return self._set_class(CssStylesIcon.CssSmallIcon)

  def small_right(self):
    """  """
    return self._set_class(CssStylesIcon.CssSmallIconRight)

  def small_red(self):
    """  """
    return self._set_class(CssStylesIcon.CssSmallIconRed)

  def out(self):
    """  """
    return self._set_class(CssStylesIcon.CssOutIcon)

  def big(self):
    """  """
    return self._set_class(CssStylesIcon.CssBigIcon)




from epyk.core.html import Html
from epyk.core.js import JsUtils
from epyk.core.js.packages import JsQuery
from epyk.core.js.html import JsHtmlJqueryUI

# The list of CSS classes
from epyk.core.css.styles import GrpChart
from epyk.core.html.options import OptSparkline


class Sparklines(Html.Html):
  requirements = ('jquery-sparklines', )
  name = 'Sparkline'

  def __init__(self, report, data, chart_type, title, width, height, options):
    super(Sparklines, self).__init__(report, data, css_attrs={'width': width, 'height': height})
    self._jsStyles, self.title = {"type": chart_type}, None
    if title is not None:
      self.title = self._report.ui.title(title, level=4)
      self.title.options.managed = False
    self.__options = OptSparkline.OptionsSparkLine(self, options or {})

  @property
  def style(self):
    """
    Description:
    ------------

    :rtype: GrpChart.ClassBSpartlines
    """
    if self._styleObj is None:
      self._styleObj = GrpChart.ClassBSpartlines(self)
    return self._styleObj

  @property
  def options(self):
    """
    Description:
    -----------
    Property to set all the possible object for a button

    :rtype: OptSparkline.OptionsSparkLine
    """
    return self.__options

  @property
  def dom(self):
    """
    Javascript Functions

    Return all the Javascript functions defined for an HTML Component.
    Those functions will use plain javascript by default.

    :return: A Javascript Dom object

    :rtype: JsHtmlJqueryUI.JsHtmlSparkline
    """
    if self._dom is None:
      self._dom = JsHtmlJqueryUI.JsHtmlSparkline(self, report=self._report)
    return self._dom

  def click(self, jsFnc, profile=False, source_event=None):
    """
    Description:
    ------------
    When a user clicks on a sparkline, a sparklineClick event is generated.
    The event object contains a property called "sparklines" that holds an array of the sparkline objects under the mouse at the time of the click.
    For non-composite sparklines, this array will have just one entry.

    Related Pages:

			https://omnipotent.net/jquery.sparkline/#interactive

    Attributes:
    ----------
    :param jsFnc:
    :param profile:
    """
    self.onReady("%s.bind('sparklineClick', function(event) { %s })" % (self.dom.jquery.varId, JsUtils.jsConvertFncs(jsFnc, toStr=True)))
    return self

  def hover(self, jsFnc, profile=False, source_event=None):
    """
    Description:
    ------------
    When the mouse moves over a different value in a sparkline a sparklineRegionChange event is generated.
    This can be useful to hook in an alternate tooltip library.

    Related Pages:

      https://omnipotent.net/jquery.sparkline/#interactive

    Attributes:
    ----------
    :param jsFnc:
    :param profile:
    """
    self.onReady("%s.bind('sparklineRegionChange', function(event) { %s })" % (self.dom.jquery.varId, JsUtils.jsConvertFncs(jsFnc, toStr=True)))
    return self

  @property
  def _js__builder__(self):
    return '%s.sparkline(data, options)' % JsQuery.decorate_var("htmlObj", convert_var=False)

  def __str__(self):
    # Javascript builder is mandatory for this object
    self._report._props.setdefault('js', {}).setdefault("builders", []).append(self.refresh())
    if self.title is not None:
      return "<div style='display:inline-block;text-align:center'>%s<span %s></span></div>" % (self.title, self.get_attrs(pyClassNames=self.style.get_classes()))

    return "<span %s>Loading..</span>" % self.get_attrs(pyClassNames=self.style.get_classes())




from epyk.core.html import Html
from epyk.core.html.options import OptNet

from epyk.core.js.objects import JsComponents
from epyk.core.js import JsUtils

# The list of CSS classes
from epyk.core.css.styles import GrpClsNetwork


class Comments(Html.Html):
  name = 'Comment'

  def __init__(self, report, recordSet, width, height, htmlCode, options, profile):
    super(Comments, self).__init__(report, recordSet, css_attrs={"width": width, 'height': height}, htmlCode=htmlCode, profile=profile)
    self.__options = OptNet.OptionsChat(self, options)
    self.css({'padding': '5px'})
    self.input = None
    self.counter = report.ui.texts.span("0", width=(None, 'px'))
    self.counter.options.managed = False
    self.counter.attr["name"] = "count"
    self.counter.css({"display": "inline-block", "margin": 0, "padding": 0, "cursor": "pointer"})
    if not self.options.readonly:
      self.input = report.ui.input()
      self.input.options.managed = False
      self.input.style.css.text_align = 'left'

  @property
  def options(self):
    """
    Description:
    ------------

    :rtype: OptNet.OptionsChat
    """
    return self.__options

  @property
  def js(self):
    """
    Description:
    -----------

    Attributes:
    ----------
    :return: A Javascript Dom object

    :rtype: JsComponents.Chat
    """
    if self._js is None:
      self._js = JsComponents.Chat(self, report=self._report)
    return self._js

  @property
  def _js__builder__(self):
    return '''
        var feed = document.createElement("p");
        feed.style.margin = "0 0 5px 0";
        if(options.showdown){var converter = new showdown.Converter(options.showdown); data = converter.makeHtml(data.trim())};
        feed.innerHTML = data; htmlObj.querySelector("div").prepend(feed);

        var now = new Date();
        var dateStringWithTime = moment(now).format('YYYY-MM-DD HH:mm:ss');
        var dateNews = document.createElement("p");
        dateNews.style.margin = 0;
        dateNews.style.fontWeight = 'bold';
        dateNews.innerHTML = dateStringWithTime;
        htmlObj.querySelector("div").prepend(dateNews);
        '''

  def enter(self, jsFncs, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param jsFncs:
    :param profile:
    """
    if self.options.readonly:
      self.options.readonly = False
      self.input = self._report.ui.input()
      self.input.options.managed = False
      self.input.style.css.text_align = 'left'
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self.input.enter(jsFncs + [
      self.js.add(self.input.dom.content),
      self.input.dom.empty()])
    return self

  def send(self, socket, channel=None, jsFncs=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param socket:
    :param channel:
    :param jsFncs:
    :param profile:
    """
    if self.options.readonly:
      self.options.readonly = False
      self.input = self._report.ui.input()
      self.input.options.managed = False
      self.input.style.css.text_align = 'left'
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self.input.enter(jsFncs + [
      socket.emit(channel or 'message', self.input.dom.content),
      self.input.dom.empty()])
    return self

  def subscribe(self, socket, channel, data=None, options=None, jsFncs=None, profile=False):
    """
    Description:
    ------------
    Subscribe to a socket channel.
    Data received from the socket are defined as a dictionary with a field data.

    The content of data will be used by this component.

    Related Pages:

      https://timepicker.co/options/

    Attributes:
    ----------
    :param socket: Socket. A python socket object
    :param channel: String. The channel on which events will be received
    """
    if data is None:
      data = socket.message
    jsFncs = jsFncs if jsFncs is not None else []
    socket.on(channel, [self.js.add(data)] + jsFncs)
    return self

  def __str__(self):
    return '''
      <div %(attr)s>
        <span>%(counter)s Comments <i style="margin:0 5px 0 20px;cursor:pointer;display:inline-block" class="fas fa-sort-amount-up"></i>Sort by</span>
        %(inputTag)s
        <div class='scroll_content' style="height:%(height)spx;margin:0;padding:5px 0;overflow:auto">
          <div name="comms"></div>
        </div>
      </div>
      ''' % {'attr': self.get_attrs(pyClassNames=self.style.get_classes()), "counter": self.counter.html(),
             'height': int(self.css("height")[:-2]) - 70,
             'inputTag': '' if self.input is None else self.input.html()}


class Bot(Html.Html):
  """

  Three main states for the bot:
    - Message
    - Question
    - Advice / Information

  """
  name = 'Bot'

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(Bot, self).__init__(report, [], css_attrs={"width": width}, htmlCode=htmlCode, profile=profile)
    self.css({"text-align": 'right', "position": 'fixed', "bottom": 0, 'margin': '10px', "height": "80px",
              "padding": "5px",  "z-index": 200})

  @property
  def style(self):
    """
    Description:
    ------------
    Property to the CSS Style of the component

    :rtype: GrpClsNetwork.ClassNetworkBot
    """
    if self._styleObj is None:
      self._styleObj = GrpClsNetwork.ClassNetworkBot(self)
    return self._styleObj

  def __str__(self):
    return '''
       <div %(attr)s>
          <div class="speech-bubble-ds">
            <p>Hello, I am Roxanne. How can I help you!</p>
            <div class="speech-bubble-ds__arrow"></div>
          </div>
          <div style="background-repeat:no-repeat;height:80px;width:80px;display:inline-block;background-image:url('')">&nbsp;</div>
       </div>   
    ''' % {'attr': self.get_attrs(), "name": 'Test'}


class Chat(Html.Html):
  name = 'Chat'

  def __init__(self, report, recordSet, width, height, htmlCode, options, profile):
    super(Chat, self).__init__(report, recordSet, css_attrs={"width": width, 'height': height}, htmlCode=htmlCode, profile=profile)
    self.__options = OptNet.OptionsChat(self, options)
    self.css({'padding': '5px'})
    self.input = None
    self.counter = report.ui.texts.span("0", width=(None, 'px'))
    self.counter.options.managed = False
    self.counter.attr["name"] = "count"
    self.counter.css({"display": "none", "margin": 0, "padding": 0, "cursor": "pointer"})
    if not self.options.readonly:
      self.input = report.ui.input()
      self.input.options.managed = False
      self.input.style.css.text_align = 'left'

  @property
  def options(self):
    """
    Description:
    ------------

    :rtype: OptNet.OptionsChat
    """
    return self.__options

  @property
  def js(self):
    """
    Description:
    -----------

    Attributes:
    ----------
    :return: A Javascript Dom object

    :rtype: JsComponents.Chat
    """
    if self._js is None:
      self._js = JsComponents.Chat(self, report=self._report)
    return self._js

  @property
  def _js__builder__(self):
    return '''
      var feed = document.createElement("p");
      feed.style.margin = "0 0 5px 0";
      if(options.showdown){var converter = new showdown.Converter(options.showdown); data = converter.makeHtml(data.trim())};
      feed.innerHTML = data; htmlObj.querySelector("div").prepend(feed);
      
      var now = new Date();
      var dateStringWithTime = moment(now).format('YYYY-MM-DD HH:mm:ss');
      var dateNews = document.createElement("p");
      dateNews.style.margin = 0;
      dateNews.style.fontWeight = 'bold';
      dateNews.innerHTML = dateStringWithTime;
      htmlObj.querySelector("div").prepend(dateNews);
      '''

  def enter(self, jsFncs, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param jsFncs:
    :param profile:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self.input.enter(jsFncs + [
      self.js.add(self.input.dom.content),
      self.input.dom.empty()])
    return self

  def send(self, socket, channel=None, jsFncs=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param socket:
    :param channel:
    :param jsFncs:
    :param profile:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self.input.enter(jsFncs + [
      socket.emit(channel or 'message', self.input.dom.content),
      self.input.dom.empty()])
    return self

  def subscribe(self, socket, channel, data=None, options=None, jsFncs=None, profile=False):
    """
    Description:
    ------------
    Subscribe to a socket channel.
    Data received from the socket are defined as a dictionary with a field data.

    The content of data will be used by this component.

    Related Pages:

      https://timepicker.co/options/

    Attributes:
    ----------
    :param socket: Socket. A python socket object
    :param channel: String. The channel on which events will be received
    """
    if data is None:
      data = socket.message
    jsFncs = jsFncs if jsFncs is not None else []
    socket.on(channel, [self.js.add(data)] + jsFncs)
    return self

  def __str__(self):
    return '''
      <div %(attr)s>
        %(inputTag)s
        <div style="margin:0;padding:5px 0;height:%(height)spx;overflow:auto"></div>
        %(counter)s
      </div>
      ''' % {'attr': self.get_attrs(pyClassNames=self.style.get_classes()), 'htmlCode': self.htmlCode, 'height': int(self.css("height")[:-2]) - 70,
             'inputTag': '' if self.input is None else self.input.html(), "counter": self.counter.html()}


class Alert(Html.Html):
  requirements = ('bootstrap', )
  name = 'Alert'

  def __init__(self, report, value, width, height, htmlCode, options, profile):
    super(Alert, self).__init__(report, value, css_attrs={"width": width, 'height': height}, htmlCode=htmlCode, profile=profile)
    self.__options = OptNet.OptionsAlert(self, options)
    self.css({"padding": '5px', 'position': 'fixed', 'top': '20px', 'right': '20px'})

  @property
  def options(self):
    """
    Description:
    ------------

    :rtype: OptNet.OptionsAlert
    """
    return self.__options

  @property
  def _js__builder__(self):
    return '''
      var feed = document.createElement("p");
      options.classes.forEach(function(cls){ feed.classList.add(cls) });
      if(options.showdown){var converter = new showdown.Converter(options.showdown); data = converter.makeHtml(data.trim())};
      if (htmlObj.style.display != 'block'){htmlObj.innerHTML = ""; };
      feed.innerHTML = data; htmlObj.appendChild(feed);
      var s = htmlObj.style; s.opacity = 1; htmlObj.style.display = 'block';
      (function fade(){(s.opacity-=.1)<0?s.display="none": setTimeout(fade, options.time)})();
      '''

  def __str__(self):
    return "<div %s></div>" % self.get_attrs(pyClassNames=self.style.get_classes())


class News(Html.Html):
  name = 'News'

  def __init__(self, report, value, width, height, htmlCode, options, profile):
    super(News, self).__init__(report, value, css_attrs={"width": width, 'height': height}, htmlCode=htmlCode, profile=profile)
    #self.css({"padding": '5px', 'position': 'fixed', 'border': '1px solid %s' % self._report.theme.success[1],
    #          "background": self._report.theme.greys[0], 'bottom': '20px', 'right': '20px'})
    self.__options = OptNet.OptionsNews(self, options)
    self._report.jsImports.add('moment')

  @property
  def _js__builder__(self):
    return '''
    var feed = document.createElement("p");
    feed.style.margin = "0 0 5px 0";
    if(options.showdown){var converter = new showdown.Converter(options.showdown); data = converter.makeHtml(data.trim())};
    feed.innerHTML = data; htmlObj.prepend(feed);
    
    if (options.dated){
      var now = new Date();
      var dateStringWithTime = moment(now).format('YYYY-MM-DD HH:mm:ss');
      var dateNews = document.createElement("p");
      dateNews.style.margin = 0;
      dateNews.style.fontWeight = 'bold';
      dateNews.innerHTML = dateStringWithTime;
      htmlObj.prepend(dateNews); }
    '''

  @property
  def js(self):
    """
    Description:
    -----------

    Attributes:
    ----------
    :return: A Javascript Dom object

    :rtype: JsComponents.News
    """
    if self._js is None:
      self._js = JsComponents.News(self, report=self._report)
    return self._js

  @property
  def options(self):
    """
    Description:
    ------------

    :rtype: OptNet.OptionsNews
    """
    return self.__options

  def __str__(self):
    return "<div %s></div>" % self.get_attrs(pyClassNames=self.style.get_classes())


class Room(Html.Html):
  name = 'room'

  def __init__(self, report, img, width, height, htmlCode, options, profile):
    super(Room, self).__init__(report, "", css_attrs={"width": width, 'height': height}, htmlCode=htmlCode, profile=profile)
    self.__options = OptNet.OptionsNews(self, options)
    color = 'green'
    self.css({"padding": '5px', 'position': 'fixed', 'bottom': '10px', 'right': '20px', 'border-radius': '30px',
              'border': '1px solid %s' % color, 'background-repeat': 'no-repeat', 'z-index': 100,
              'background-position': 'center', 'background-size': 'cover', 'cursor': 'pointer',
              'background-image': 'url(%s)' % img})
    self.status = self._report.ui.div()
    self.status.options.managed = False
    self.status.attr['name'] = "status"
    self.status.css({"width": "15px", "height": "15px", "background": color, "border-radius": "20px",
                     "border": "1px solid %s" % color, "bottom": "0px", "position": "absolute", "right": 0})
    self.dots = self._report.ui.div(".")
    self.dots.attr['name'] = "dots"
    self.dots.options.managed = False
    self.dots.css({"bottom": "-5px", "position": "absolute", "left": "-25px", 'font-size': '30px', 'display': 'none'})

    keyframe_name = "dots"
    self.dots.style.css.animation = "%s 1s infinite" % keyframe_name
    attrs = {"0%, 20%": {"color": "rgba(0,0,0,0)", "text-shadow": ".25em 0 0 rgba(0,0,0,0),.5em 0 0 rgba(0,0,0,0)"},
             "40%": {"text-shadow": ".25em 0 0 rgba(0,0,0,0),.5em 0 0 rgba(0,0,0,0)"},
             "60%": {"text-shadow": ".25em 0 0 %s,.5em 0 0 rgba(0,0,0,0)" % report.theme.greys[-1]},
             "80%, 100%": {"text-shadow": ".25em 0 0 %s,.5em 0 0 %s" % (report.theme.greys[-1], report.theme.greys[-1])}}
    self.dots.style.css_class.keyframes(keyframe_name, attrs)

  @property
  def js(self):
    """
    Description:
    -----------

    Attributes:
    ----------
    :return: A Javascript Dom object

    :rtype: JsComponents.Chat
    """
    if self._js is None:
      self._js = JsComponents.Room(self, report=self._report)
    return self._js

  def __str__(self):
    return "<div %s>%s%s</div>" % (self.get_attrs(pyClassNames=self.style.get_classes()), self.dots.html(), self.status.html())


class DropFile(Html.Html):
  requirements = ('font-awesome',)
  name, inputType = 'Drop File Area', "file"

  def __init__(self, report, vals, tooltip, options, profile):
    super(DropFile, self).__init__(report, vals, profile=profile)
    self.__options = OptNet.OptionFiles(self, options)
    self.tooltip(tooltip, location='bottom')
    self.css({"display": "inline-block", "width": '100%', 'text-align': 'center', 'margin': '0 5px',
              'border': "1px dashed %s" % report.theme.colors[-1]})

  @property
  def options(self):
    """
    Description:
    ------------
    Specific options for the files components

    :rtype: OptNet.OptionFiles
    """
    return self.__options

  def transfer(self, url):
    """
    Description:
    -----------
    Create a Ajax transfer to a distant server

    Attributes:
    ----------
    :param url: String. The transfer end point on the server
    """
    # TODO add if else statement for the allowed and forbidden extensions
    post = self._report.js.post(url, self._report.js.objects.get("form_data"))
    return post

  def drop(self, jsFncs, preventDefault=True, profile=False):
    """
    Description:
    -----------
    Add a drag and drop property to the element

    Usage::

    Attributes:
    ----------
    :param jsFncs: List. The Javascript series of functions
    :param preventDefault: Boolean. Prevent default on the JavaScript event
    :param profile: Boolean. Profiling

    :return: Return self to allow the chaining
    """
    dft_fnc = ""
    if preventDefault:
      dft_fnc = self._report.js.objects.event.preventDefault()
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    form_data = self._report.js.data.formdata()
    jsFncs = [form_data.new("form_data"),  form_data.append("file", self._report.js.objects.event.dataTransfer.files[0])] + jsFncs
    str_fncs = JsUtils.jsConvertFncs(["var data = %s" % self._report.js.objects.event.dataTransfer.files] + jsFncs, toStr=True)
    self.attr["ondragover"] = "(function(event){%s})(event)" % dft_fnc
    self.on("drop", ["%s; %s; return false" % (dft_fnc, str_fncs)])
    return self

  def __str__(self):
    return '''
      <div %(strAttr)s>
        <b><i class="fas fa-cloud-upload-alt" style="font-size:20px"></i>&nbsp;&nbsp;</b>
      </div>
      <input id="%(htmlCode)s_report" style="display:none;"/>
      ''' % {'htmlCode': self.htmlCode, 'strAttr': self.get_attrs(pyClassNames=self.style.get_classes())}




from epyk.core.data.DataClass import DataClass
from epyk.core.data.DataClass import DataEnum


class EnumAlign(DataEnum):

  js_conversion = True

  def left(self):
    return self.set()

  def right(self):
    return self.set()

  def center(self):
    return self.set()


class EnumOrientation(DataEnum):

  js_conversion = True

  def top(self):
    return self.set()

  def bottom(self):
    return self.set()


class Enum3dStyles(DataEnum):
  js_conversion = True

  def bar(self):
    return self.set()

  def bar_color(self):
    return self.set("bar-color")

  def bar_size(self):
    return self.set("bar-size")

  def dot(self):
    return self.set()

  def dot_line(self):
    return self.set("dot-line")

  def dot_color(self):
    return self.set("dot-color")

  def dot_size(self):
    return self.set("dot-size")

  def line(self):
    return self.set()

  def grid(self):
    return self.set()

  def surface(self):
    return self.set()


class EnumNodeShapes(DataEnum):

  js_conversion = True

  def dot(self):
    return self.set()


class EnumPointsStyle(DataEnum):

  js_conversion = True

  def square(self):
    return self.set()

  def circle(self):
    return self.set()


class OptionsTime(DataClass):

  @property
  def scale(self):
    return self._attrs["scale"]

  @scale.setter
  def scale(self, val):
    self._attrs["scale"] = val

  @property
  def step(self):
    return self._attrs["step"]

  @step.setter
  def step(self, val):
    self._attrs["step"] = val


class OptionsBackgroundColor(DataClass):

  @property
  def fill(self):
    """
    Description:
    -----------
    The chart fill color, as an HTML color string.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["fill"]

  @fill.setter
  def fill(self, val):
    self._attrs["fill"] = val

  @property
  def stroke(self):
    """
    Description:
    -----------
    The color of the chart border, as an HTML color string.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["stroke"]

  @stroke.setter
  def stroke(self, val):
    self._attrs["stroke"] = val

  @property
  def strokeWidth(self):
    """
    Description:
    -----------
    The border width, in pixels.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["strokeWidth"]

  @strokeWidth.setter
  def strokeWidth(self, val):
    self._attrs["strokeWidth"] = val


class OptionAxis(DataClass):

  @property
  def icon(self):
    return self._attrs["icon"]

  @icon.setter
  def icon(self, val):
    self._attrs["icon"] = val

  @property
  def left(self):
    """

    :rtype: OptionRange
    """
    return self.has_attribute(OptionRange)

  @property
  def right(self):
    """

    :rtype: OptionRange
    """
    return self.has_attribute(OptionRange)

  @property
  def showMajorLabels(self):
    return self._attrs["showMajorLabels"]

  @showMajorLabels.setter
  def showMajorLabels(self, val):
    self._attrs["showMajorLabels"] = val

  @property
  def showMinorLabels(self):
    return self._attrs["showMinorLabels"]

  @showMinorLabels.setter
  def showMinorLabels(self, val):
    self._attrs["showMinorLabels"] = val

  @property
  def visible(self):
    return self._attrs["visible"]

  @visible.setter
  def visible(self, val):
    self._attrs["visible"] = val


class OptionRange(DataClass):

  @property
  def min(self):
    return self._attrs["min"]

  @min.setter
  def min(self, val):
    self._attrs["min"] = val

  @property
  def max(self):
    return self._attrs["max"]

  @max.setter
  def max(self, val):
    self._attrs["max"] = val


class OptionAxisPosition(DataClass):

  @property
  def range(self):
    """

    :rtype: OptionRange
    """
    return self.has_attribute(OptionRange)


class OptionsCameraPosition(DataClass):

  @property
  def horizontal(self):
    """
    Description:
    -----------
    Value in radians. It can have any value, but is normally in the range of 0 and 2*Pi.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["horizontal"]

  @horizontal.setter
  def horizontal(self, val):
    self._attrs["horizontal"] = val

  @property
  def vertical(self):
    """
    Description:
    -----------
    Value in radians between 0 and 0.5*Pi.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["vertical"]

  @vertical.setter
  def vertical(self, val):
    self._attrs["vertical"] = val

  @property
  def distance(self):
    """
    Description:
    -----------
    The (normalized) distance from the camera to the center of the graph, in the range of 0.71 to 5.0. A larger distance puts the graph further away, making it smaller.

    https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["distance"]

  @distance.setter
  def distance(self, val):
    self._attrs["distance"] = val


class OptionsInterpolation(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggle the interpolation.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def parametrization(self):
    """
    Description:
    -----------
    Define the type of parametrizaion for the catmullRom interpolation. Example 7 shows the different parametrizations.
    The options are 'centripetal' (best results), 'chordal' and 'uniform'. Uniform is the computationally cheapest variant.
    If interpolation is disabled, linear interpolation is used.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["parametrization"]

  @parametrization.setter
  def parametrization(self, val):
    self._attrs["parametrization"] = val


class OptionContent(DataClass):

  @property
  def background(self):
    return self._attrs["background"]

  @background.setter
  def background(self, val):
    self._attrs["background"] = val

  @property
  def color(self):
    return self._attrs["color"]

  @color.setter
  def color(self, val):
    self._attrs["color"] = val

  @property
  def border(self):
    return self._attrs["border"]

  @border.setter
  def border(self, val):
    self._attrs["border"] = val

  @property
  def borderRadius(self):
    return self._attrs["borderRadius"]

  @borderRadius.setter
  def borderRadius(self, val):
    self._attrs["borderRadius"] = val

  @property
  def padding(self):
    return self._attrs["padding"]

  @padding.setter
  def padding(self, val):
    self._attrs["padding"] = val

  @property
  def boxShadow(self):
    return self._attrs["boxShadow"]

  @boxShadow.setter
  def boxShadow(self, val):
    self._attrs["boxShadow"] = val


class OptionLine(DataClass):

  @property
  def height(self):
    return self._attrs["height"]

  @height.setter
  def height(self, val):
    self._attrs["height"] = val

  @property
  def width(self):
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val

  @property
  def borderLeft(self):
    return self._attrs["borderLeft"]

  @borderLeft.setter
  def borderLeft(self, val):
    self._attrs["borderLeft"] = val


class OptionDot(DataClass):

  @property
  def height(self):
    return self._attrs["height"]

  @height.setter
  def height(self, val):
    self._attrs["height"] = val

  @height.setter
  def height(self, val):
    self._attrs["height"] = val

  @property
  def width(self):
    return self._attrs["width"]

  @property
  def border(self):
    return self._attrs["border"]

  @border.setter
  def border(self, val):
    self._attrs["border"] = val

  @property
  def borderRadius(self):
    return self._attrs["borderRadius"]

  @borderRadius.setter
  def borderRadius(self, val):
    self._attrs["borderRadius"] = val


class OptionsTooltipStyle(DataClass):

  @property
  def content(self):
    """

    :rtype: OptionContent
    """
    return self.has_attribute(OptionContent)

  @property
  def line(self):
    """

    :rtype: OptionLine
    """
    return self.has_attribute(OptionLine)

  @property
  def dot(self):
    """

    :rtype: OptionDot
    """
    return self.has_attribute(OptionDot)


class OptionsdrawPoints(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggles the drawing of the datapoints.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def onRender(self):
    """
    Description:
    -----------
    Defines a render function for every datapoint. If a group has no drawPoints.onRender callback, the graph2d drawPoints.onRender callback will be used. If neither is defined, the datapoint will be rendered according to the group setting of drawPoints.enabled. This callback must return true if the datapoint should be rendered, otherwise false.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["onRender"]

  @onRender.setter
  def onRender(self, val):
    self._attrs["onRender"] = val

  @property
  def size(self):
    """
    Description:
    -----------
    Determine the size at which the data points are drawn.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["size"]

  @size.setter
  def size(self, val):
    self._attrs["size"] = val

  @property
  def style(self):
    """

    :rtype: EnumPointsStyle
    """
    return self.has_attribute(EnumPointsStyle)


class OptionBarChart(DataClass):

  @property
  def align(self):
    """
    Description:
    -----------
    The alignment of the bars with regards to the coordinate. The options are 'left', 'right' or 'center'.

    https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: EnumAlign
    """
    return self.has_attribute(EnumAlign)

  @property
  def sideBySide(self):
    """
    Description:
    -----------
    If two datapoints of a barchart overlap, they are drawn over eachother by default.
    If sideBySide is set to true, they will be drawn side by side, within the same width as a single bar.. See example 10 for more information. When using groups, see example 11.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["sideBySide"]

  @sideBySide.setter
  def sideBySide(self, val):
    self._attrs["sideBySide"] = val

  @property
  def width(self):
    """
    Description:
    -----------
    The width of the bars

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val

  @property
  def minWidth(self):
    """
    Description:
    -----------
    The minimum width of the bars in pixels: by default the bars get smaller while zooming out to prevent overlap, this value is the minimum width of the bar. Default behavior (when minWidth is not set) is 10% of the bar width.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["minWidth"]

  @minWidth.setter
  def minWidth(self, val):
    self._attrs["minWidth"] = val


class OptionsLegendPosition(DataClass):

  @property
  def visible(self):
    """
    Description:
    -----------
    Both axis, left and right, have a corresponding legend. This toggles the visibility of the legend that is coupled with the left axis.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["visible"]

  @visible.setter
  def visible(self, val):
    self._attrs["visible"] = val

  @property
  def position(self):
    """
    Description:
    -----------
    Determine the position of the legend coupled to the left axis. Options are 'top-left', 'top-right', 'bottom-left' or 'bottom-right'.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["position"]

  @position.setter
  def position(self, val):
    self._attrs["position"] = val


class OptionsLegend(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggle the legend.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def icons(self):
    """
    Description:
    -----------
    Show automatically generated icons on the legend.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["icons"]

  @icons.setter
  def icons(self, val):
    self._attrs["icons"] = val

  @property
  def left(self):
    """
    Description:
    -----------

    :https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: OptionsLegendPosition
    """
    return self.has_attribute(OptionsLegendPosition)

  @property
  def right(self):
    """
    Description:
    -----------

    https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: OptionsLegendPosition
    """
    return self.has_attribute(OptionsLegendPosition)


class Options2D(DataClass):

  @property
  def barChart(self):
    """

    :rtype: OptionBarChart
    """
    return self.has_attribute(OptionBarChart)

  @property
  def configure(self):
    """
    Description:
    -----------
    When true, a configurator is loaded where all configuration options of the Graph2d can be changed live.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["configure"]

  @configure.setter
  def configure(self, val):
    self._attrs["configure"] = val

  @property
  def clickToUse(self):
    return self._attrs["clickToUse"]

  @clickToUse.setter
  def clickToUse(self, val):
    self._attrs["clickToUse"] = val

  @property
  def defaultGroup(self):
    """
    Description:
    -----------
    This is the label for the default, ungrouped items when shown in a legend.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["defaultGroup"]

  @defaultGroup.setter
  def defaultGroup(self, val):
    self._attrs["defaultGroup"] = val

  @property
  def drawPoints(self):
    """
    Description:
    -----------

    https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: OptionsdrawPoints
    """
    return self.has_attribute(OptionsdrawPoints)

  @property
  def dataAxis(self):
    """
    Description:
    -----------

    https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: OptionAxis
    """
    return self.has_attribute(OptionAxis)

  @property
  def interpolation(self):
    """

    https://visjs.github.io/vis-timeline/docs/graph2d/

    :rtype: OptionsInterpolation
    """
    return self.has_attribute(OptionsInterpolation)

  @property
  def end(self):
    return self._attrs["end"]

  @end.setter
  def end(self, val):
    self._attrs["end"] = val

  @property
  def graphHeight(self):
    """
    Description:
    -----------
    This is the height of the graph SVG canvas.
    If it is larger than the height of the outer frame, you can drag up and down the vertical direction as well as the usual horizontal direction.

    https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["graphHeight"]

  @graphHeight.setter
  def graphHeight(self, val):
    self._attrs["graphHeight"] = val

  @property
  def height(self):
    """

    :return:
    """
    return self._attrs["height"]

  @height.setter
  def height(self, val):
    if isinstance(val, int):
      val = "%spx" % val
    # self.graphHeight = val
    self._attrs["height"] = val

  @property
  def legend(self):
    """

    :rtype: OptionsLegend
    """
    return self.has_attribute[OptionsLegend]

  @property
  def max(self):
    return self._attrs["max"]

  @max.setter
  def max(self, val):
    self._attrs["max"] = val

  @property
  def maxHeight(self):
    return self._attrs["maxHeight"]

  @maxHeight.setter
  def maxHeight(self, val):
    self._attrs["maxHeight"] = val

  @property
  def maxMinorChars(self):
    return self._attrs["maxMinorChars"]

  @maxMinorChars.setter
  def maxMinorChars(self, val):
    self._attrs["maxMinorChars"] = val

  @property
  def min(self):
    return self._attrs["min"]

  @min.setter
  def min(self, val):
    self._attrs["min"] = val

  @property
  def minHeight(self):
    return self._attrs["minHeight"]

  @minHeight.setter
  def minHeight(self, val):
    self._attrs["minHeight"] = val

  @property
  def moveable(self):
    return self._attrs["moveable"]

  @moveable.setter
  def moveable(self, val):
    self._attrs["moveable"] = val

  @property
  def orientation(self):
    """

    :rtype: EnumOrientation
    """
    return self.has_attribute(EnumOrientation)

  @property
  def shaded(self):
    """

    :rtype: EnumOrientation
    """
    return self.has_attribute(EnumOrientation)

  @property
  def showCurrentTime(self):
    return self._attrs["showCurrentTime"]

  @showCurrentTime.setter
  def showCurrentTime(self, val):
    self._attrs["showCurrentTime"] = val

  @property
  def showCustomTime(self):
    return self._attrs["showCustomTime"]

  @showCustomTime.setter
  def showCustomTime(self, val):
    self._attrs["showCustomTime"] = val

  @property
  def sort(self):
    return self._attrs["sort"]

  @sort.setter
  def sort(self, val):
    self._attrs["sort"] = val

  @property
  def start(self):
    return self._attrs["start"]

  @start.setter
  def start(self, val):
    self._attrs["start"] = val

  @property
  def style(self):
    return self._attrs["style"]

  @style.setter
  def style(self, val):
    self._attrs["style"] = val

  @property
  def width(self):
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val

  @property
  def zoomable(self):
    return self._attrs["zoomable"]

  @zoomable.setter
  def zoomable(self, val):
    self._attrs["zoomable"] = val

  @property
  def zoomMax(self):
    return self._attrs["zoomMax"]

  @zoomMax.setter
  def zoomMax(self, val):
    self._attrs["zoomMax"] = val

  @property
  def zoomMin(self):
    return self._attrs["zoomMin"]

  @zoomMin.setter
  def zoomMin(self, val):
    self._attrs["zoomMin"] = val

  @property
  def timeAxis(self):
    return self.sub_data("timeAxis", OptionsTime)

  @property
  def managed(self):
    """
    Description:
    ------------
    """
    return self.get(True)

  @managed.setter
  def managed(self, bool):
    self.set(bool)


class Options3D(DataClass):

  @property
  def animationInterval(self):
    """
    Description:
    -----------
    The animation interval in milliseconds. This determines how fast the animation runs.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["animationInterval"]

  @animationInterval.setter
  def animationInterval(self, val):
    self._attrs["animationInterval"] = val

  @property
  def animationPreload(self):
    """
    Description:
    -----------
    If false, the animation frames are loaded as soon as they are requested.
    if animationPreload is true, the graph will automatically load all frames in the background, resulting in a smoother animation as soon as all frames are loaded. The load progress is shown on screen.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["animationPreload"]

  @animationPreload.setter
  def animationPreload(self, val):
    self._attrs["animationPreload"] = val

  @property
  def animationAutoStart(self):
    """
    Description:
    -----------
    If true, the animation starts playing automatically after the graph is created.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["animationAutoStart"]

  @animationAutoStart.setter
  def animationAutoStart(self, val):
    self._attrs["animationAutoStart"] = val

  @property
  def axisColor(self):
    """
    Description:
    -----------
    The color of the axis lines and the text along the axis.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["axisColor"]

  @axisColor.setter
  def axisColor(self, val):
    self._attrs["axisColor"] = val

  @property
  def autoResize(self):
    return self._attrs["autoResize"]

  @autoResize.setter
  def autoResize(self, val):
    self._attrs["autoResize"] = val

  @property
  def backgroundColor(self):
    return self.sub_data("backgroundColor", OptionsBackgroundColor)

  @property
  def cameraPosition(self):
    """

    :rtype: OptionsCameraPosition
    """
    return self.has_attribute(OptionsCameraPosition)

  @property
  def ctrlToZoom(self):
    return self._attrs["ctrlToZoom"]

  @ctrlToZoom.setter
  def ctrlToZoom(self, val):
    self._attrs["ctrlToZoom"] = val

  @property
  def dataColor(self):
    return self.sub_data("dataColor", OptionsBackgroundColor)

  @property
  def dotSizeRatio(self):
    """
    Description:
    -----------
    Ratio of the size of the dots with respect to the width of the graph.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["dotSizeRatio"]

  @dotSizeRatio.setter
  def dotSizeRatio(self, val):
    self._attrs["dotSizeRatio"] = val

  @property
  def dotSizeMinFraction(self):
    """
    Description:
    -----------
    Size of minimum-value dot as a fraction of dotSizeRatio. Applicable when using style dot-size.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["dotSizeMinFraction"]

  @dotSizeMinFraction.setter
  def dotSizeMinFraction(self, val):
    self._attrs["dotSizeMinFraction"] = val

  @property
  def dotSizeMaxFraction(self):
    """
    Description:
    -----------
    Size of maximum-value dot as a fraction of dotSizeRatio. Applicable when using style dot-size.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["dotSizeMaxFraction"]

  @dotSizeMaxFraction.setter
  def dotSizeMaxFraction(self, val):
    self._attrs["dotSizeMaxFraction"] = val

  @property
  def filterLabel(self):
    return self._attrs["filterLabel"]

  @filterLabel.setter
  def filterLabel(self, val):
    self._attrs["filterLabel"] = val

  @property
  def gridColor(self):
    """
    Description:
    -----------
    The color of the grid lines.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["gridColor"]

  @gridColor.setter
  def gridColor(self, val):
    self._attrs["gridColor"] = val

  @property
  def height(self):
    """
    Description:
    -----------
    The height of the graph in pixels or as a percentage.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["height"]

  @height.setter
  def height(self, val):
    if isinstance(val, float):
      val = "%spx" % val
    self._attrs["height"] = val

  @property
  def keepAspectRatio(self):
    """
    Description:
    -----------
    If keepAspectRatio is true, the x-axis and the y-axis keep their aspect ratio.
    If false, the axes are scaled such that they both have the same, maximum width.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["keepAspectRatio"]

  @keepAspectRatio.setter
  def keepAspectRatio(self, val):
    self._attrs["keepAspectRatio"] = val

  @property
  def rotateAxisLabels(self):
    """
    Description:
    -----------
    If rotateAxisLabels is true, the x-axis and y-axis labels will rotate with the graph. Useful with long label values.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["rotateAxisLabels"]

  @rotateAxisLabels.setter
  def rotateAxisLabels(self, val):
    self._attrs["rotateAxisLabels"] = val

  @property
  def showAnimationControls(self):
    """
    Description:
    -----------
    If true, animation controls are created at the bottom of the Graph.
    The animation controls consists of buttons previous, start/stop, next, and a slider showing the current frame.
    Only applicable when the provided data contains an animation.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showAnimationControls"]

  @showAnimationControls.setter
  def showAnimationControls(self, val):
    self._attrs["showAnimationControls"] = val

  @property
  def showGrayBottom(self):
    """
    Description:
    -----------
    If true, draw the bottom side of the surface in gray.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showGrayBottom"]

  @showGrayBottom.setter
  def showGrayBottom(self, val):
    self._attrs["showGrayBottom"] = val

  @property
  def showGrid(self):
    """
    Description:
    -----------
    If true, grid lines are drawn in the x-y surface (the bottom of the 3d graph).

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showGrid"]

  @showGrid.setter
  def showGrid(self, val):
    self._attrs["showGrid"] = val

  @property
  def showXAxis(self):
    """
    Description:
    -----------
    If true, X axis and X axis labels are drawn

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showXAxis"]

  @showXAxis.setter
  def showXAxis(self, val):
    self._attrs["showXAxis"] = val

  @property
  def showYAxis(self):
    """
    Description:
    -----------
    If true, Y axis and Y axis labels are drawn.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showYAxis"]

  @showYAxis.setter
  def showYAxis(self, val):
    self._attrs["showYAxis"] = val

  @property
  def showZAxis(self):
    """
    Description:
    -----------
    If true, Z axis and Z axis labels are drawn.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showZAxis"]

  @showZAxis.setter
  def showZAxis(self, val):
    self._attrs["showZAxis"] = val

  @property
  def showPerspective(self):
    """
    Description:
    -----------
    If true, the graph is drawn in perspective: points and lines which are further away are drawn smaller.
    Note that the graph currently does not support a gray colored bottom side when drawn in perspective.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showPerspective"]

  @showPerspective.setter
  def showPerspective(self, val):
    self._attrs["showPerspective"] = val

  @property
  def showLegend(self):
    """
    Description:
    -----------
    If true, a legend is drawn for the graph (if the graph type supports it).
    By default a legend is drawn for dot and dot-color style graphs.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showLegend"]

  @showLegend.setter
  def showLegend(self, val):
    self._attrs["showLegend"] = val

  @property
  def showShadow(self):
    """
    Description:
    -----------
    Show shadow on the graph.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showShadow"]

  @showShadow.setter
  def showShadow(self, val):
    self._attrs["showShadow"] = val

  @property
  def showSurfaceGrid(self):
    """
    Description:
    -----------
    If true, grid lines are drawn on the surface of the graph itself (only effective if style: 'surface'.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["showSurfaceGrid"]

  @showSurfaceGrid.setter
  def showSurfaceGrid(self, val):
    self._attrs["showSurfaceGrid"] = val

  @property
  def style(self):
    """
    Description:
    -----------
    The style of the 3d graph

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html

    :rtype: Enum3dStyles
    """
    return self.has_attribute(Enum3dStyles)

  @property
  def tooltip(self):
    """
    Description:
    -----------
    Show a tooltip showing the values of the hovered data point.
    The contents of the tooltip can be customized by providing a callback function as tooltip.
    In this case the function is called with an object containing parameters x, y, z, and data (the source JS object for the point) as an argument, and must return a string which may contain HTML.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["tooltip"]

  @tooltip.setter
  def tooltip(self, val):
    self._attrs["tooltip"] = val

  @property
  def tooltipDelay(self):
    """
    Description:
    -----------
    The delay time (in ms) for the tooltip to appear when the mouse cursor hovers over an x-y grid tile.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["tooltipDelay"]

  @tooltipDelay.setter
  def tooltipDelay(self, val):
    self._attrs["tooltipDelay"] = val

  @property
  def tooltipStyle(self):
    """

    :rtype: OptionsTooltipStyle
    """
    return self.has_attribute(OptionsTooltipStyle)

  @property
  def valueMax(self):
    """
    Description:
    -----------
    The maximum value for the value-axis. Only available in combination with the styles dot-color and dot-size.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["valueMax"]

  @valueMax.setter
  def valueMax(self, val):
    self._attrs["valueMax"] = val

  @property
  def valueMin(self):
    """
    Description:
    -----------
    The minimum value for the value-axis. Only available in combination with the styles dot-color and dot-size.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["valueMin"]

  @valueMin.setter
  def valueMin(self, val):
    self._attrs["valueMin"] = val

  @property
  def verticalRatio(self):
    """
    Description:
    -----------
    A value between 0.1 and 1.0.
    This scales the vertical size of the graph When keepAspectRatio is set to false, and verticalRatio is set to 1.0, the graph will be a cube.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["verticalRatio"]

  @verticalRatio.setter
  def verticalRatio(self, val):
    self._attrs["verticalRatio"] = val

  @property
  def width(self):
    """
    Description:
    -----------
    The width of the graph in pixels or as a percentage.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    if isinstance(val, float):
      val = "%spx" % val
    self._attrs["width"] = val

  @property
  def xCenter(self):
    """
    Description:
    -----------
    The horizontal center position of the graph, as a percentage or in pixels.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xCenter"]

  @xCenter.setter
  def xCenter(self, val):
    self._attrs["xCenter"] = val

  @property
  def xMax(self):
    """
    Description:
    -----------
    The maximum value for the x-axis. If not set, the largest value for x in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xMax"]

  @xMax.setter
  def xMax(self, val):
    self._attrs["xMax"] = val

  @property
  def xMin(self):
    """
    Description:
    -----------
    The minimum value for the x-axis. If not set, the smallest value for x in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xMin"]

  @xMin.setter
  def xMin(self, val):
    self._attrs["xMin"] = val

  @property
  def xBarWidth(self):
    """
    Description:
    -----------
    The width of bars in x direction. By default, the width is equal to the smallest distance between the data points.
    Only applicable for styles 'bar' and 'bar-color'.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xBarWidth"]

  @xBarWidth.setter
  def xBarWidth(self, val):
    self._attrs["xBarWidth"] = val

  @property
  def xValueLabel(self):
    """
    Description:
    -----------
    A function for custom formatting of the labels along the x-axis, for example function (x) {return (x * 100) + '%'}.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xValueLabel"]

  @xValueLabel.setter
  def xValueLabel(self, val):
    self._attrs["xValueLabel"] = val

  @property
  def xStep(self):
    """
    Description:
    -----------
    Step size for the grid on the x-axis.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["xStep"]

  @xStep.setter
  def xStep(self, val):
    self._attrs["xStep"] = val

  @property
  def yBarWidth(self):
    """
    Description:
    -----------
    The width of bars in y direction. By default, the width is equal to the smallest distance between the data points.
    Only applicable for styles 'bar' and 'bar-color'.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yBarWidth"]

  @yBarWidth.setter
  def yBarWidth(self, val):
    self._attrs["yBarWidth"] = val

  @property
  def yCenter(self):
    """
    Description:
    -----------
    The vertical center position of the graph, as a percentage or in pixels.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yCenter"]

  @yCenter.setter
  def yCenter(self, val):
    self._attrs["yCenter"] = val

  @property
  def yMax(self):
    """
    Description:
    -----------
    The maximum value for the y-axis. If not set, the largest value for y in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yMax"]

  @yMax.setter
  def yMax(self, val):
    self._attrs["yMax"] = val

  @property
  def yMin(self):
    """
    Description:
    -----------
    The minimum value for the y-axis. If not set, the smallest value for y in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yMin"]

  @yMin.setter
  def yMin(self, val):
    self._attrs["yMin"] = val

  @property
  def yValueLabel(self):
    """
    Description:
    -----------
    A function for custom formatting of the labels along the y-axis, for example function (y) {return (y * 100) + '%'}.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yValueLabel"]

  @yValueLabel.setter
  def yValueLabel(self, val):
    self._attrs["yValueLabel"] = val

  @property
  def yStep(self):
    """
    Description:
    -----------
    Step size for the grid on the y-axis.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["yStep"]

  @yStep.setter
  def yStep(self, val):
    self._attrs["yStep"] = val

  @property
  def zValueLabel(self):
    """
    Description:
    -----------
    A function for custom formatting of the labels along the z-axis, for example function (z) {return (z * 100) + '%'}

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["zValueLabel"]

  @zValueLabel.setter
  def zValueLabel(self, val):
    self._attrs["zValueLabel"] = val

  @property
  def zMax(self):
    """
    Description:
    -----------
    The maximum value for the z-axis. If not set, the largest value for z in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["zMax"]

  @zMax.setter
  def zMax(self, val):
    self._attrs["zMax"] = val

  @property
  def zMin(self):
    """
    Description:
    -----------
    The minimum value for the z-axis. If not set, the smallest value for z in the data set is used.

    Related Pages:

			https://visjs.github.io/vis-graph3d/docs/graph3d/index.html
    """
    return self._attrs["zMin"]

  @zMin.setter
  def zMin(self, val):
    self._attrs["zMin"] = val

  @property
  def managed(self):
    """
    Description:
    ------------
    """
    return self.get(True)

  @managed.setter
  def managed(self, bool):
    self.set(bool)


class OptionsEditable(DataClass):

  @property
  def add(self):
    """
    Description:
    -----------
    If true, new items can be created by double tapping an empty space in the Timeline. See section Editing Items for a detailed explanation.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["add"]

  @add.setter
  def add(self, val):
    self._attrs["add"] = val

  @property
  def remove(self):
    """
    Description:
    -----------
    If true, items can be deleted by first selecting them, and then clicking the delete button on the top right of the item. See section Editing Items for a detailed explanation.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["remove"]

  @remove.setter
  def remove(self, val):
    self._attrs["remove"] = val

  @property
  def updateGroup(self):
    """
    Description:
    -----------
    If true, items can be dragged from one group to another. Only applicable when the Timeline has groups. See section Editing Items for a detailed explanation.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["updateGroup"]

  @updateGroup.setter
  def updateGroup(self, val):
    self._attrs["updateGroup"] = val

  @property
  def updateTime(self):
    """
    Description:
    -----------
    If true, items can be dragged to another moment in time. See section Editing Items for a detailed explanation.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["updateTime"]

  @updateTime.setter
  def updateTime(self, val):
    self._attrs["updateTime"] = val

  @property
  def overrideItems(self):
    """
    Description:
    -----------
    If true, item specific editable properties are overridden by timeline settings

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["overrideItems"]

  @overrideItems.setter
  def overrideItems(self, val):
    self._attrs["overrideItems"] = val


class OptionsTimeline(DataClass):

  @property
  def autoResize(self):
    """
    Description:
    -----------
    If true, the Timeline will automatically detect when its container is resized, and redraw itself accordingly.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["autoResize"]

  @autoResize.setter
  def autoResize(self, val):
    self._attrs["autoResize"] = val

  @property
  def editable(self):
    """

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/

    :rtype: OptionsEditable
    """
    return self.has_attribute(OptionsEditable)

  @editable.setter
  def editable(self, val):
    self._attrs["editable"] = val

  @property
  def end(self):
    """
    Description:
    -----------
    The initial end date for the axis of the timeline. If not provided, the latest date present in the items set is taken as end date.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/graph2d/
    """
    return self._attrs["end"]

  @end.setter
  def end(self, val):
    self._attrs["end"] = val

  @property
  def clickToUse(self):
    """
    Description:
    -----------
    When a Timeline is configured to be clickToUse, it will react to mouse and touch events only when active. When active, a blue shadow border is displayed around the Timeline. The Timeline is set active by clicking on it, and is changed to inactive again by clicking outside the Timeline or by pressing the ESC key.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["clickToUse"]

  @clickToUse.setter
  def clickToUse(self, val):
    self._attrs["clickToUse"] = val

  @property
  def start(self):
    """
    Description:
    -----------
    The initial start date for the axis of the timeline. If not provided, the earliest date present in the events is taken as start date.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["start"]

  @start.setter
  def start(self, val):
    self._attrs["start"] = val

  @property
  def stack(self):
    """
    Description:
    -----------
    If true (default), items will be stacked on top of each other such that they do not overlap.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["stack"]

  @stack.setter
  def stack(self, val):
    self._attrs["stack"] = val

  @property
  def stackSubgroups(self):
    """
    Description:
    -----------
    If true (default), subgroups will be stacked on top of each other such that they do not overlap.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["stackSubgroups"]

  @stackSubgroups.setter
  def stackSubgroups(self, val):
    self._attrs["stackSubgroups"] = val

  @property
  def multiselect(self):
    """
    Description:
    -----------
    If true, multiple items can be selected using ctrl+click, shift+click, or by holding items. Only applicable when option selectable is true.

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["multiselect"]

  @multiselect.setter
  def multiselect(self, val):
    self._attrs["multiselect"] = val

  @property
  def sequentialSelection(self):
    """
    Description:
    -----------
    If true, then only sequential items are allowed to be selected (no gaps) when multiselect is true

    Related Pages:

			https://visjs.github.io/vis-timeline/docs/timeline/
    """
    return self._attrs["sequentialSelection"]

  @sequentialSelection.setter
  def sequentialSelection(self, val):
    self._attrs["sequentialSelection"] = val

  @property
  def managed(self):
    """
    Description:
    ------------
    """
    return self.get(True)

  @managed.setter
  def managed(self, bool):
    self.set(bool)


class OptionLabel(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggle the scaling of the label on or off. If this option is not defined, it is set to true if any of the properties in this object are defined.

    Related Pages:

			https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def min(self):
    """
    Description:
    -----------
    The minimum font-size used for labels when scaling.

    Related Pages:

			https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["min"]

  @min.setter
  def min(self, val):
    self._attrs["min"] = val

  @property
  def max(self):
    """
    Description:
    -----------
    The maximum font-size used for labels when scaling.

    Related Pages:

			https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["max"]

  @max.setter
  def max(self, val):
    self._attrs["max"] = val

  @property
  def maxVisible(self):
    """
    Description:
    -----------
    When zooming in, the font is drawn larger as well.
    You can limit the perceived font size using this option.
    If set to 30, the font will never look larger than size 30 zoomed at 100%.

    Related Pages:

			https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["maxVisible"]

  @maxVisible.setter
  def maxVisible(self, val):
    self._attrs["maxVisible"] = val

  @property
  def drawThreshold(self):
    """
    Description:
    -----------
    When zooming out, the font will be drawn smaller. This defines a lower limit for when the font is drawn.
    When using font scaling, you can use this together with the maxVisible to first show labels of important edges when zoomed out and only show the rest when zooming in.

    Related Pages:

			https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["drawThreshold"]

  @drawThreshold.setter
  def drawThreshold(self, val):
    self._attrs["drawThreshold"] = val


class OptionNodeScaling(DataClass):

  @property
  def min(self):
    return self._attrs["min"]

  @min.setter
  def min(self, val):
    self._attrs["min"] = val

  @property
  def max(self):
    return self._attrs["max"]

  @max.setter
  def max(self, val):
    self._attrs["max"] = val

  @property
  def label(self):
    """

    :rtype: OptionLabel
    """
    return self.has_attribute(OptionLabel)


class OptionFont(DataClass):

  @property
  def size(self):
    return self._attrs["size"]

  @size.setter
  def size(self, val):
    self._attrs["size"] = val

  @property
  def face(self):
    return self._attrs["face"]

  @face.setter
  def face(self, val):
    self._attrs["face"] = val


class OptionNode(DataClass):

  @property
  def shape(self):
    """

    :rtype: EnumNodeShapes
    """
    return self.has_attribute(EnumNodeShapes)

  @property
  def scaling(self):
    """

    :rtype: OptionNodeScaling
    """
    return self.has_attribute(OptionNodeScaling)

  @property
  def font(self):
    """

    :rtype: OptionFont
    """
    return self.has_attribute(OptionFont)


class OptionPhysicsBarnesHut(DataClass):

  @property
  def springLength(self):
    return self._attrs["springLength"]

  @springLength.setter
  def springLength(self, val):
    self._attrs["springLength"] = val


class OptionPhysics(DataClass):

  @property
  def stabilization(self):
    return self._attrs["stabilization"]

  @stabilization.setter
  def stabilization(self, val):
    self._attrs["stabilization"] = val

  @property
  def physics(self):
    """

    :rtype: OptionPhysicsBarnesHut
    """
    return self.has_attribute(OptionPhysicsBarnesHut)


class OptionColor(DataClass):

  @property
  def color(self):
    return self._attrs["color"]

  @color.setter
  def color(self, val):
    self._attrs["color"] = val

  @property
  def highlight(self):
    return self._attrs["highlight"]

  @highlight.setter
  def highlight(self, val):
    self._attrs["highlight"] = val

  @property
  def hover(self):
    return self._attrs["hover"]

  @hover.setter
  def hover(self, val):
    self._attrs["hover"] = val


class OptionInteraction(DataClass):

  @property
  def hover(self):
    return self._attrs["hover"]

  @hover.setter
  def hover(self, val):
    self._attrs["hover"] = val

  @property
  def hideEdgesOnDrag(self):
    return self._attrs["hideEdgesOnDrag"]

  @hideEdgesOnDrag.setter
  def hideEdgesOnDrag(self, val):
    self._attrs["hideEdgesOnDrag"] = val

  @property
  def tooltipDelay(self):
    return self._attrs["tooltipDelay"]

  @tooltipDelay.setter
  def tooltipDelay(self, val):
    self._attrs["tooltipDelay"] = val


class OptionSmooth(DataEnum):

  def continuous(self):
    return self.set()

  def cubicBezier(self):
    return self.set()


class OptionShadow(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggle the casting of shadows. If this option is not defined, it is set to true if any of the properties in this object are defined.

    https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def color(self):
    """
    Description:
    -----------
    The color size of the shadow as a string. Supported formats are 'rgb(255,255,255)', 'rgba(255,255,255,1)' and '#FFFFFF'.

    https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["color"]

  @color.setter
  def color(self, val):
    self._attrs["color"] = val

  @property
  def size(self):
    """
    Description:
    -----------
    The blur size of the shadow.

    https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["size"]

  @size.setter
  def size(self, val):
    self._attrs["size"] = val

  @property
  def x(self):
    """
    Description:
    -----------
    The x offset.

    https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["x"]

  @x.setter
  def x(self, val):
    self._attrs["x"] = val

  @property
  def y(self):
    """
    Description:
    -----------
    The y offset.

    https://visjs.github.io/vis-network/docs/network/edges.html
    """
    return self._attrs["y"]

  @y.setter
  def y(self, val):
    self._attrs["y"] = val


class OptionEdge(DataClass):

  @property
  def color(self):
    """

    :rtype: OptionColor
    """
    return self.has_attribute(OptionColor)

  @property
  def hoverWidth(self):
    return self._attrs["hoverWidth"]

  @hoverWidth.setter
  def hoverWidth(self, val):
    self._attrs["hoverWidth"] = val

  @property
  def interaction(self):
    """

    :rtype: OptionInteraction
    """
    return self.has_attribute(OptionInteraction)

  @property
  def width(self):
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val

  @property
  def smooth(self):
    """

    :rtype: OptionSmooth
    """
    return self.has_attribute(OptionSmooth)

  @property
  def shadow(self):
    """

    :rtype: OptionShadow
    """
    return self.has_attribute(OptionShadow)


class OPtionsHierarchical(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Toggle the usage of the hierarchical layout system.
    If this option is not defined, it is set to true if any of the properties in this object are defined.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val

  @property
  def levelSeparation(self):
    """
    Description:
    -----------
    The distance between the different levels.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["levelSeparation"]

  @levelSeparation.setter
  def levelSeparation(self, val):
    self._attrs["levelSeparation"] = val

  @property
  def nodeSpacing(self):
    """
    Description:
    -----------
    Minimum distance between nodes on the free axis. This is only for the initial layout.
    If you enable physics, the node distance there will be the effective node distance

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["nodeSpacing"]

  @nodeSpacing.setter
  def nodeSpacing(self, val):
    self._attrs["nodeSpacing"] = val

  @property
  def treeSpacing(self):
    """
    Description:
    -----------
    Distance between different trees (independent networks). This is only for the initial layout.
    If you enable physics, the repulsion model will denote the distance between the trees.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["treeSpacing"]

  @treeSpacing.setter
  def treeSpacing(self, val):
    self._attrs["treeSpacing"] = val

  @property
  def blockShifting(self):
    """
    Description:
    -----------
    Method for reducing whitespace. Can be used alone or together with edge minimization.
    Each node will check for whitespace and will shift it's branch along with it for as far as it can, respecting the nodeSpacing on any level.
    This is mainly for the initial layout. If you enable physics, the layout will be determined by the physics. This will greatly speed up the stabilization time though!

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["blockShifting"]

  @blockShifting.setter
  def blockShifting(self, val):
    self._attrs["blockShifting"] = val

  @property
  def edgeMinimization(self):
    """
    Description:
    -----------
    Method for reducing whitespace. Can be used alone or together with block shifting.
    Enabling block shifting will usually speed up the layout process.
    Each node will try to move along its free axis to reduce the total length of it's edges.
    This is mainly for the initial layout. If you enable physics, the layout will be determined by the physics.
    This will greatly speed up the stabilization time though!

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["edgeMinimization"]

  @edgeMinimization.setter
  def edgeMinimization(self, val):
    self._attrs["edgeMinimization"] = val

  @property
  def parentCentralization(self):
    """
    Description:
    -----------
    When true, the parents nodes will be centered again after the layout algorithm has been finished.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["parentCentralization"]

  @parentCentralization.setter
  def parentCentralization(self, val):
    self._attrs["parentCentralization"] = val


class OptionLayout(DataClass):

  @property
  def randomSeed(self):
    """
    Description:
    -----------
    When NOT using the hierarchical layout, the nodes are randomly positioned initially.
    This means that the settled result is different every time. If you provide a random seed manually,
    the layout will be the same every time.
    Ideally you try with an undefined seed, reload until you are happy with the layout and use the getSeed() method to ascertain the seed.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["randomSeed"]

  @randomSeed.setter
  def randomSeed(self, val):
    self._attrs["randomSeed"] = val

  @property
  def improvedLayout(self):
    """
    Description:
    -----------
    When enabled, the network will use the Kamada Kawai algorithm for initial layout.
    For networks larger than 100 nodes, clustering will be performed automatically to reduce the amount of nodes.
    This can greatly improve the stabilization times. If the network is very interconnected (no or few leaf nodes),
    this may not work and it will revert back to the old method. Performance will be improved in the future

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["improvedLayout"]

  @improvedLayout.setter
  def improvedLayout(self, val):
    self._attrs["improvedLayout"] = val

  @property
  def clusterThreshold(self):
    """
    Description:
    -----------
    Cluster threshold to which improvedLayout applies.

    https://visjs.github.io/vis-network/docs/network/layout.html#
    """
    return self._attrs["clusterThreshold"]

  @clusterThreshold.setter
  def clusterThreshold(self, val):
    self._attrs["clusterThreshold"] = val

  @property
  def hierarchical(self):
    """
    Description:
    -----------

    https://visjs.github.io/vis-network/docs/network/layout.html#

    :rtype: OPtionsHierarchical
    """
    return self.has_attribute(OPtionsHierarchical)


class OptionManipulation(DataClass):

  @property
  def enabled(self):
    return self._attrs["enabled"]

  @enabled.setter
  def enabled(self, val):
    self._attrs["enabled"] = val


class OptionsNetwork(DataClass):

  @property
  def autoResize(self):
    """
    Description:
    -----------
    If true, the Network will automatically detect when its container is resized, and redraw itself accordingly.
    If false, the Network can be forced to repaint after its container has been resized using the function redraw() and setSize().

    https://visjs.github.io/vis-network/docs/network/
    """
    return self._attrs["autoResize"]

  @autoResize.setter
  def autoResize(self, val):
    self._attrs["autoResize"] = val

  @property
  def width(self):
    """
    Description:
    -----------
    the width of the canvas. Can be in percentages or pixels (ie. '400px').

    https://visjs.github.io/vis-network/docs/network/
    """
    return self._attrs["width"]

  @width.setter
  def width(self, val):
    self._attrs["width"] = val

  @property
  def height(self):
    """
    Description:
    -----------
    the height of the canvas. Can be in percentages or pixels (ie. '400px').

    https://visjs.github.io/vis-network/docs/network/
    """
    return self._attrs["height"]

  @height.setter
  def height(self, val):
    self._attrs["height"] = val

  @property
  def locale(self):
    """
    Description:
    -----------
    Select the locale. By default, the language is English

    https://visjs.github.io/vis-network/docs/network/
    """
    return self._attrs["locale"]

  @locale.setter
  def locale(self, val):
    self._attrs["locale"] = val

  @property
  def nodes(self):
    """
    Description:
    -----------
    All options in this object are explained in the nodes module.

    https://visjs.github.io/vis-network/docs/network/

    :rtype: OptionNode
    """
    return self.has_attribute(OptionNode)

  @property
  def edges(self):
    """
    Description:
    -----------
    All options in this object are explained in the edges module.

    https://visjs.github.io/vis-network/docs/network/

    :rtype: OptionEdge
    """
    return self.has_attribute(OptionEdge)

  @property
  def layout(self):
    """
    Description:
    -----------
    https://visjs.github.io/vis-network/docs/network/

    All options in this object are explained in the layout module.

    :rtype: OptionLayout
    """
    return self.has_attribute(OptionLayout)

  @property
  def physics(self):
    """
    Description:
    -----------
    All options in this object are explained in the physics module.

    https://visjs.github.io/vis-network/docs/network/

    :rtype: OptionPhysics
    """
    return self.has_attribute(OptionPhysics)

  @property
  def interaction(self):
    """
    Description:
    -----------
    All options in this object are explained in the interaction module.

    https://visjs.github.io/vis-network/docs/network/

    :rtype: OptionInteraction
    """
    return self.has_attribute(OptionInteraction)

  @property
  def manipulation(self):
    """
    Description:
    -----------
    All options in this object are explained in the manipulation module.

    https://visjs.github.io/vis-network/docs/network/

    :rtype: OptionManipulation
    """
    return self.has_attribute(OptionManipulation)

  @property
  def managed(self):
    """
    Description:
    ------------
    """
    return self.get(True)

  @managed.setter
  def managed(self, bool):
    self.set(bool)




from epyk.core.data.DataClass import DataClass

from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsObjects


class Line(DataClass):

  @property
  def color(self):
    """
    Description:
    -----------

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get('#F66')

  @color.setter
  def color(self, color):
    self.set(color)

  @property
  def width(self):
    """
    Description:
    -----------

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get(1)

  @width.setter
  def width(self, num):
    self.set(num)


class Sync(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Enable or disable syncing of crosshairs between charts of the same group

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html
    """
    return self.get(True)

  @enabled.setter
  def enabled(self, bool):
    self.set(bool)

  @property
  def group(self):
    """
    Description:
    -----------
    Limit crosshair syncing to charts belonging to the same 'group'

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get(1)

  @group.setter
  def group(self, num):
    self.set(num)

  @property
  def suppressTooltips(self):
    """
    Description:
    -----------
    Allows for suppressing tooltips when showing a synced crosshair

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get(True)

  @suppressTooltips.setter
  def suppressTooltips(self, bool):
    self.set(bool)


class Zoom(DataClass):

  @property
  def enabled(self):
    """
    Description:
    -----------
    Enable or disable zooming by drag and drop

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get(True)

  @enabled.setter
  def enabled(self, bool):
    self.set(bool)

  @property
  def zoomboxBackgroundColor(self):
    """
    Description:
    -----------
    Background color of the zoombox

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get('rgba(66,133,244,0.2)')

  @zoomboxBackgroundColor.setter
  def zoomboxBackgroundColor(self, color):
    self.set(color)

  @property
  def zoomboxBorderColor(self):
    """
    Description:
    -----------
    Border color of the zoombox

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get('#48F')

  @zoomboxBorderColor.setter
  def zoomboxBorderColor(self, color):
    self.set(color)

  @property
  def zoomButtonText(self):
    """
    Description:
    -----------
    Text of the button to reset the chart to original axis ranges.

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get('Reset Zoom')

  @zoomButtonText.setter
  def zoomButtonText(self, text):
    self.set(text)

  @property
  def zoomButtonClass(self):
    """
    Description:
    -----------
    Class of the button to reset the chart to original axis ranges.

    Related Pages:

			https://www.npmjs.com/package/chartjs-plugin-crosshair
    """
    return self.get('reset-zoom')

  @zoomButtonClass.setter
  def zoomButtonClass(self, text):
    self.set(text)


class Crosshair(DataClass):

  @property
  def line(self):
    """
    Description:
    -----------

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html
    """
    return self.sub_data("line", Line)

  @property
  def sync(self):
    """
    Description:
    -----------
    The plugin allows for syncing crosshairs over multiple charts

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html
    """
    return self.sub_data("sync", Sync)

  @property
  def zoom(self):
    """
    Description:
    -----------
    The plugin allows for horizontal zooming by clicking and dragging over the chart.

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html
    """
    return self.sub_data("zoom", Zoom)

  @property
  def snapToDataPoint(self):
    """
    Description:
    -----------
    The plugin allows snapping to datapoints when used with line charts

    Related Pages:

      https://canvasjs.com/docs/charts/chart-options/axisx/crosshair/snap-to-datapoint/
    """
    return self.get(True)

  @snapToDataPoint.setter
  def snapToDataPoint(self, bool):
    self.set(bool)

  def beforeZoom(self, jsFncs):
    """
    Description:
    -----------
    Called before zooming, return false to prevent the zoom

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._attrs["beforeZoom"] = JsObjects.JsVoid("function(start, end) { %s }" % JsUtils.jsConvertFncs(jsFncs, toStr=True))

  def afterZoom(self, jsFncs):
    """
    Description:
    -----------
    Called after zooming, can for example be used for reloading data at a higher resolution

    Related Pages:

			https://chartjs-plugin-crosshair.netlify.app/options.html

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._attrs["afterZoom"] = JsObjects.JsVoid("function(start, end) { %s }" % JsUtils.jsConvertFncs(jsFncs, toStr=True))





from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsObjects


class SocketIO(object):

  def __init__(self, htmlCode=None, src=None):
    """
    Description:
    ------------
    """
    if src is not None:
      src.jsImports.add('socket.io')
    self._src = src
    self._selector = htmlCode or "socket_%s" % id(self)

  @property
  def message(self):
    """

    """
    return JsObjects.JsObject.JsObject.get("data")

  def send(self, msg):
    """
    Description:
    ------------
    This will send an event called message(built in) to our client, four seconds after the client connects.
    The send function on socket object associates the 'message' event.

    Usage::

      https://www.tutorialspoint.com/socket.io/socket.io_event_handling.htm

    Attributes:
    ----------
    :param msg:
    """
    msg = JsUtils.jsConvertData(msg, None)
    return JsObjects.JsVoid("%s.send(%s)" % (self._selector, msg))

  def join(self, roomId):
    """
    Description:
    ------------

    Usage::

      https://www.tutorialspoint.com/socket.io/socket.io_rooms.htm

    Attributes:
    ----------
    :param roomId:
    """
    roomId = JsUtils.jsConvertData(roomId, None)
    return JsObjects.JsVoid("%s.join(%s)" % (self._selector, roomId))

  def inRoom(self, roomId, eventType, jsData=None):
    """
    Description:
    ------------

    Usage::

      https://www.tutorialspoint.com/socket.io/socket.io_rooms.htm

    Attributes:
    ----------
    :param roomId: String. The room identifier
    """
    jsData = JsUtils.jsConvertData(jsData or {}, None)
    eventType = JsUtils.jsConvertData(eventType, None)
    roomId = JsUtils.jsConvertData(roomId, None)
    return JsObjects.JsVoid("%s.in(%s).emit(%s, %s)" % (self._selector, roomId, eventType, jsData))

  def leave(self, roomId):
    """
    Description:
    ------------

    Usage::

      https://www.tutorialspoint.com/socket.io/socket.io_rooms.htm

    Attributes:
    ----------
    :param roomId: String. The room identifier
    """
    roomId = JsUtils.jsConvertData(roomId, None)
    return JsObjects.JsVoid("%s.leave(%s)" % (self._selector, roomId))

  def connect(self, url=None, port=None, namespace=None, from_config=None):
    """
    Description:
    ------------
    This function will automatically add the socket to the page object.
    This must be defined first in order to be used in the various components

    Attributes:
    ----------
    :param url: String. The server url
    :param port: Integer. The server port
    :param namespace: String. Optional. The server namespace (or room)
    :param from_config: Python Object. An internal Server configuration object (page.js.server())
    """
    if from_config is not None:
      self._src._props['js']['builders'].add("var %s = io.connect(%s)" % (self._selector, from_config.address))
      return JsObjects.JsVoid("var %s = io.connect(%s)" % (self._selector, from_config.address))

    elif url is None:
      self._src._props['js']['builders'].add("var %s = io.connect()" % self._selector)
      return JsObjects.JsVoid("var %s = io.connect()" % self._selector)

    if namespace is None:
      self._src._props['js']['builders'].add("var %s = io.connect('%s:%s')" % (self._selector, url, port))
      return JsObjects.JsVoid("var %s = io.connect('%s:%s')" % (self._selector, url, port))

    self._src._props['js']['builders'].add("var %s = io.connect('%s:%s/%s')" % (self._selector, url, port, namespace))
    return JsObjects.JsVoid("var %s = io.connect('%s:%s/%s')" % (self._selector, url, port, namespace))

  def on(self, eventType, jsFncs):
    """
    Description:
    ------------

    Usage::

      https://www.tutorialspoint.com/socket.io/socket.io_event_handling.htm

    Attributes:
    ----------
    :param eventType:
    :param jsFncs:

    :return: self to allow the chaining
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    eventType = JsUtils.jsConvertData(eventType, None)
    self._src.js.onReady("%s.on(%s, function(data) {%s})" % (self._selector, eventType, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def emit(self, eventType, jsData=None):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param eventType:
    :param jsData:
    """
    jsData = JsUtils.jsConvertData(jsData or {}, None)
    eventType = JsUtils.jsConvertData(eventType, None)
    return JsObjects.JsVoid("%s.emit(%s, %s)" % (self._selector, eventType, jsData))



"""
CSS Style module for the Dates components
"""

from epyk.core.css.styles.classes import CssStyle
from epyk.core.css import Defaults_css


class CssDatePickerUI(CssStyle.Style):
  classname = "ui-datepicker"

  def customize(self):
    self.css({"background": self.rptObj.theme.colors[0], "font-size": Defaults_css.font(), 'width': 'auto'})


class CssDatePicker(CssStyle.Style):
  _attrs = {'font-family': 'inherit', 'border': 'none', 'cursor': 'pointer', 'margin': 0,
            'padding': '2px', 'display': 'inline-block', 'border-radius': '5px', 'text-align': 'center'}
  _focus = {'outline': 0}

  def customize(self):
    self.css({"background": self.rptObj.theme.colors[0], "color": self.rptObj.theme.greys[-1],
              'border': '1px solid %s' % self.rptObj.theme.colors[0]})
    self.hover.css({'color': self.rptObj.theme.colors[-1]})


class CssDatesDatePickerHeader(CssStyle.Style):
  classname = "ui-widget-header"

  def customize(self):
    self.css({"background": self.rptObj.theme.success[1]})


class CssDatesTimePicker(CssStyle.Style):
  _attrs = {'margin': 0}
  classname = "ui-timepicker-standard"

  def customize(self):
    self.css({'color': self.rptObj.theme.greys[-1], "background": self.rptObj.theme.colors[0],
              "font-size": Defaults_css.font()})


class CssDatesTimePickerState(CssStyle.Style):
  classname = "ui-timepicker-standard .ui-state-hover"

  def customize(self):
    self.css({"background": self.rptObj.theme.colors[0]})




"""

"""

from epyk.core.css.catalogs import Catalog

from epyk.core.css.styles.classes import CssStylesHref


class CatalogLink(Catalog.CatalogGroup):

  def no_decoration(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefNoDecoration)

  def label_dates(self):
    """  """
    return self._set_class(CssStylesHref.CssLabelDates)

  def menu(self):
    """  """
    return self._set_class(CssStylesHref.CssHreftMenu)

  def sub_menu(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefSubMenu)

  def sidebar(self):
    """  """
    return self._set_class(CssStylesHref.CssSideBarLinks)

  def level_1(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefContentLevel1)

  def level_2(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefContentLevel2)

  def level_3(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefContentLevel3)

  def level_4(self):
    """  """
    return self._set_class(CssStylesHref.CssHrefContentLevel4)

  def feedback(self):
    """  """
    return self._set_class(CssStylesHref.CssFeedbackLink)

  def standard(self):
    """  """
    return self._set_class(CssStylesHref.CssStandardLinks)



"""

"""

import datetime

from epyk.core.html import Html
from epyk.core.js.Imports import requires

sqlalchemy = requires("sqlalchemy", reason='Missing Package', install='sqlalchemy', source_script=__file__)


def system_user_configuration():
  return [sqlalchemy.Column('row_id', sqlalchemy.Integer, autoincrement=True, primary_key=True),
    sqlalchemy.Column('html_code', sqlalchemy.Integer, nullable=False),
    sqlalchemy.Column('report_name', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('env', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('data', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('username', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('type', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('active_flag', sqlalchemy.String, nullable=False),
    sqlalchemy.Column('lst_mod_dt', sqlalchemy.DateTime, default=datetime.datetime.utcnow(), nullable=True)]


class ConfigTable(Html.Html):
  requirements = ('tabulator', )
  name = 'Config Table'

  def __init__(self, report, htmlCode, visible, profile):
    pass

  def load(self):
    """

    :return:
    """

  def get(self):
    """

    :param htmlCode:
    :return:
    """






aggFnss = getAggFnc()
print( aggFnss['sum Over Sum Agg'].toJs({'digits': 0}))


cols = TableCols()
cols.add([1, 2], 'div', attr={'dsc': "dfgfd"})
cols.add([1, 2], 'container_class', attr={'class': "dfgfd"})
cols.add(1, 'age', attr={'steps': 100, 'start': 'white', 'end': 'pink', 'columnOrders': []})
#cols.add(2, TableRowStyle, attr={'class': "gdfgdf"})
#cols.add(2, TableRowStyle, attr={'class': "youpi"})
#cols.add(1, TableRowStyleCss, attr={'css': {'color': 'yellow'}})



from epyk.core.html import tables as html_tables

from epyk.interfaces.tables import CompTabulator
from epyk.interfaces.tables import CompDatatable
from epyk.interfaces.tables import CompTablesPlotly
from epyk.interfaces.tables import CompTableD3
from epyk.interfaces.tables import CompAgGrid
from epyk.interfaces.tables import CompPivot
from epyk.interfaces.tables import CompTableGoogle


class Tables(object):
  def __init__(self, context):
    self.context = context
    # Default table configuration per available framework
    self.datatable = self.datatables.table
    self.tabulator = self.tabulators.table
    self.plotly = self.plotlys.table
    self.aggrid = self.aggrids.table
    self.pivot = self.pivots.pivot

  @property
  def aggrids(self):
    """
    Description:
    -----------
    ag-Grid is the industry standard for JavaScript Enterprise Applications.
    Developers using ag-Grid are building applications that would not be possible if ag-Grid did not exist.

    Related Pages:

      https://www.ag-grid.com/javascript-grid/
    """
    return CompAgGrid.AgGrid(self)

  @property
  def tabulators(self):
    """
    Description:
    -----------
    Interface to the different Tabulator configurations

    Related Pages:
http://tabulator.info/
    """
    return CompTabulator.Tabulators(self)

  @property
  def google(self):
    """
    Description:
    -----------
    """
    if not getattr(self.context.rptObj, '_with_google_imports', False):
      raise Exception("Google produce must be added using for example rptObj.imports().google_products(['charts'])")

    return CompTableGoogle.Google(self)

  @property
  def pivots(self):
    """
    Description:
    -----------
    Interface to the different Pivot Table configurations

    Related Pages:
https://pivottable.js.org/examples/
    """
    return CompPivot.Pivottable(self)

  @property
  def d3(self):
    """
    Description:
    -----------
    Interface to the different Tabulator configurations

    Related Pages:

			https://github.com/d3/d3/wiki/Gallery

    :rtype: CompTableD3.D3
    """
    return CompTableD3.D3(self)

  @property
  def plotlys(self):
    """
    Interface to the different Tabulator configurations

    Related Pages:
http://tabulator.info/

    :rtype: CompTabulator.Tabulators
    """
    return CompTablesPlotly.Plotly(self)

  @property
  def datatables(self):
    """
    Description:
    -----------
    Interface to the different Datatable configurations

    Related Pages:

			https://datatables.net/

    :rtype: CompDatatable.Datatables
    """
    return CompDatatable.Datatables(self)

  def config(self, htmlCode, visible=False, profile=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param htmlCode:
    :param visible:
    :param profile:
    """
    return html_tables.HtmlTableConfig.ConfigTable(self.context.rptObj, htmlCode, visible, profile)

  def basic(self, records, cols, rows, width=(100, '%'), height=(None, 'px'), htmlCode=None, options=None, profile=None):
    """
    Description:
    -----------

    Usage::

      simple_table = rptObj.ui.tables.basic(df.to_dict("records"), cols=["COL1"], rows=["COL2"])
    simple_table.add({"COL1": "Value"})

    Attributes:
    ----------
    :param records:
    :param cols:
    :param rows:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    table = html_tables.HtmlTable.Bespoke(self.context.rptObj, records, cols, rows, width, height, htmlCode, options, profile)
    return table

  def grid(self, records, cols, rows, width=(None, '%'), height=(None, 'px'), htmlCode=None, options=None, profile=None):
    """

    Usage::

      """
    width_cells, width_rows_header = 50, 100
    for rec in records:
      for c in cols:
        if c not in rec:
          rec[c] = 0
    table = html_tables.HtmlTable.Bespoke(self.context.rptObj, records, cols, rows, width, height, htmlCode, options, profile)
    table.css({"width": "%spx" % (width_rows_header + len(cols) * width_cells)})
    # table[0][0]._vals = ""
    for i in table[1:]:
      for j in range(len(rows)):
        i[j].attr["name"] = "row_header"
        i[j].css({"width": "%spx" % width_rows_header})
      for cell in i[j:]:
        cell.attr["contenteditable"] = 'true'
        cell.css({"width": '%spx' % width_cells})
    table.style.clear()
    # Set the style for the table grid
    # table.style.add_classes.table.grid_row_header()
    table.style.add_classes.table.grid_vals()
    # table.style.add_classes.table.grid_no_header()
    return table
